#!/usr/bin/env python3

# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This script generates upb source files (e.g. *.upb.c) from all upb targets
# in Bazel BUILD file. These generate upb files are for non-Bazel build such
# as makefile and python build which cannot generate them at the build time.
#
# As an example, for the following upb target
#
#   grpc_upb_proto_library(
#     name = "grpc_health_upb",
#     deps = ["//src/proto/grpc/health/v1:health_proto_descriptor"],
#   )
#
# this will generate these upb source files at src/core/ext/upb-generated.
#
#   src/proto/grpc/health/v1/health.upb.c
#   src/proto/grpc/health/v1/health.upb.h

import argparse
import collections
import os
import shutil
import subprocess
import xml.etree.ElementTree

# Rule object representing the UPB rule of Bazel BUILD.
Rule = collections.namedtuple("Rule", "name type srcs deps proto_files")

BAZEL_BIN = "tools/bazel"


def parse_bazel_rule(elem):
    """Returns a rule from bazel XML rule."""
    srcs = []
    deps = []
    for child in elem:
        if child.tag == "list" and child.attrib["name"] == "srcs":
            for tag in child:
                if tag.tag == "label":
                    srcs.append(tag.attrib["value"])
        if child.tag == "list" and child.attrib["name"] == "deps":
            for tag in child:
                if tag.tag == "label":
                    deps.append(tag.attrib["value"])
        if child.tag == "label":
            # extract actual name for alias rules
            label_name = child.attrib["name"]
            if label_name in ["actual"]:
                actual_name = child.attrib.get("value", None)
                if actual_name:
                    # HACK: since we do a lot of transitive dependency scanning,
                    # make it seem that the actual name is a dependency of the alias rule
                    # (aliases don't have dependencies themselves)
                    deps.append(actual_name)
    return Rule(elem.attrib["name"], elem.attrib["class"], srcs, deps, [])


def get_transitive_protos(rules, t):
    que = [
        t,
    ]
    visited = set()
    ret = []
    while que:
        name = que.pop(0)
        rule = rules.get(name, None)
        if rule:
            for dep in rule.deps:
                if dep not in visited:
                    visited.add(dep)
                    que.append(dep)
            for src in rule.srcs:
                if src.endswith(".proto"):
                    ret.append(src)
    return list(set(ret))


def read_upb_bazel_rules():
    """Runs bazel query on given package file and returns all upb rules."""
    # Use a wrapper version of bazel in gRPC not to use system-wide bazel
    # to avoid bazel conflict when running on Kokoro.
    result = subprocess.check_output(
        [BAZEL_BIN, "query", "--output", "xml", "--noimplicit_deps", "//:all"]
    )
    root = xml.etree.ElementTree.fromstring(result)
    rules = [
        parse_bazel_rule(elem)
        for elem in root
        if elem.tag == "rule"
        and elem.attrib["class"]
        in [
            "upb_proto_library",
            "upb_proto_reflection_library",
        ]
    ]
    # query all dependencies of upb rules to get a list of proto files
    all_deps = [dep for rule in rules for dep in rule.deps]
    result = subprocess.check_output(
        [
            BAZEL_BIN,
            "query",
            "--output",
            "xml",
            "--noimplicit_deps",
            " union ".join("deps({0})".format(d) for d in all_deps),
        ]
    )
    root = xml.etree.ElementTree.fromstring(result)
    dep_rules = {}
    for dep_rule in (
        parse_bazel_rule(elem) for elem in root if elem.tag == "rule"
    ):
        dep_rules[dep_rule.name] = dep_rule
    # add proto files to upb rules transitively
    for rule in rules:
        if not rule.type.startswith("upb_proto_"):
            continue
        if len(rule.deps) == 1:
            rule.proto_files.extend(
                get_transitive_protos(dep_rules, rule.deps[0])
            )
    return rules


def build_upb_bazel_rules(rules):
    result = subprocess.check_output(
        [BAZEL_BIN, "build"] + [rule.name for rule in rules]
    )


def get_upb_path(proto_path, ext):
    return proto_path.replace(":", "/").replace(".proto", ext)


def get_bazel_bin_root_path(elink):
    BAZEL_BIN_ROOT = "bazel-bin/"
    if elink[0].startswith("@"):
        # external
        result = os.path.join(
            BAZEL_BIN_ROOT,
            "external",
            elink[0].replace("@", "").replace("//", ""),
        )
        if elink[1]:
            result = os.path.join(result, elink[1])
        return result
    else:
        # internal
        return BAZEL_BIN_ROOT


def get_external_link(file):
    EXTERNAL_LINKS = [
        ("@com_google_protobuf//", "src/"),
        ("@com_google_googleapis//", ""),
        ("@com_github_cncf_udpa//", ""),
        ("@com_envoyproxy_protoc_gen_validate//", ""),
        ("@envoy_api//", ""),
        ("@opencensus_proto//", ""),
    ]
    for external_link in EXTERNAL_LINKS:
        if file.startswith(external_link[0]):
            return external_link
    return ("//", "")


def copy_upb_generated_files(rules, args):
    files = {}
    for rule in rules:
        if rule.type == "upb_proto_library":
            frag = ".upb"
            output_dir = args.upb_out
        else:
            frag = ".upbdefs"
            output_dir = args.upbdefs_out
        for proto_file in rule.proto_files:
            elink = get_external_link(proto_file)
            prefix_to_strip = elink[0] + elink[1]
            if not proto_file.startswith(prefix_to_strip):
                raise Exception(
                    'Source file "{0}" in does not have the expected prefix'
                    ' "{1}"'.format(proto_file, prefix_to_strip)
                )
            proto_file = proto_file[len(prefix_to_strip) :]
            for ext in (".h", ".c"):
                file = get_upb_path(proto_file, frag + ext)
                src = os.path.join(get_bazel_bin_root_path(elink), file)
                dst = os.path.join(output_dir, file)
                files[src] = dst
    for src, dst in files.items():
        if args.verbose:
            print("Copy:")
            print("    {0}".format(src))
            print(" -> {0}".format(dst))
        os.makedirs(os.path.split(dst)[0], exist_ok=True)
        shutil.copyfile(src, dst)


parser = argparse.ArgumentParser(description="UPB code-gen from bazel")
parser.add_argument("--verbose", default=False, action="store_true")
parser.add_argument(
    "--upb_out",
    default="src/core/ext/upb-generated",
    help="Output directory for upb targets",
)
parser.add_argument(
    "--upbdefs_out",
    default="src/core/ext/upbdefs-generated",
    help="Output directory for upbdefs targets",
)


def main():
    args = parser.parse_args()
    rules = read_upb_bazel_rules()
    if args.verbose:
        print("Rules:")
        for rule in rules:
            print(
                "  name={0} type={1} proto_files={2}".format(
                    rule.name, rule.type, rule.proto_files
                )
            )
    if rules:
        build_upb_bazel_rules(rules)
        copy_upb_generated_files(rules, args)


if __name__ == "__main__":
    main()



# Copyright 2022 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The classes and predicates to assist validate test config for test cases."""
from dataclasses import dataclass
import enum
import logging
import re
from typing import Optional

from packaging import version as pkg_version

logger = logging.getLogger(__name__)


class Lang(enum.Flag):
    UNKNOWN = enum.auto()
    CPP = enum.auto()
    GO = enum.auto()
    JAVA = enum.auto()
    PYTHON = enum.auto()
    NODE = enum.auto()

    def __str__(self):
        return str(self.name).lower()

    @classmethod
    def from_string(cls, lang: str):
        try:
            return cls[lang.upper()]
        except KeyError:
            return cls.UNKNOWN


@dataclass
class TestConfig:
    """Describes the config for the test suite.

    TODO(sergiitk): rename to LangSpec and rename skips.py to lang.py.
    """

    client_lang: Lang
    server_lang: Lang
    version: Optional[str]

    def version_gte(self, another: str) -> bool:
        """Returns a bool for whether this VERSION is >= then ANOTHER version.

        Special cases:

        1) Versions "master" or "dev" are always greater than ANOTHER:
        - master > v1.999.x > v1.55.x
        - dev > v1.999.x > v1.55.x
        - dev == master

        2) Versions "dev-VERSION" behave the same as the VERSION:
        - dev-master > v1.999.x > v1.55.x
        - dev-master == dev == master
        - v1.55.x > dev-v1.54.x > v1.53.x
        - dev-v1.54.x == v1.54.x

        3) Unspecified version (self.version is None) is treated as "master".
        """
        if self.version in ("master", "dev", "dev-master", None):
            return True

        # The left side is not master, so master on the right side wins.
        if another == "master":
            return False

        # Treat "dev-VERSION" on the left side as "VERSION".
        version: str = self.version
        if version.startswith("dev-"):
            version = version[4:]
        return self._parse_version(version) >= self._parse_version(another)

    def __str__(self):
        return (
            f"TestConfig(client_lang='{self.client_lang}', "
            f"server_lang='{self.server_lang}', version={self.version!r})"
        )

    @staticmethod
    def _parse_version(version: str) -> pkg_version.Version:
        if version.endswith(".x"):
            version = version[:-2]
        return pkg_version.Version(version)


def get_lang(image_name: str) -> Lang:
    return Lang.from_string(
        re.search(r"/(\w+)-(client|server):", image_name).group(1)
    )



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests that a channel will reconnect if a connection is dropped"""

import logging
import socket
import time
import unittest

import grpc
from grpc.framework.foundation import logging_pool

from tests.unit.framework.common import bound_socket
from tests.unit.framework.common import test_constants

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x00\x00\x01"

_UNARY_UNARY = "/test/UnaryUnary"


def _handle_unary_unary(unused_request, unused_servicer_context):
    return _RESPONSE


class ReconnectTest(unittest.TestCase):
    def test_reconnect(self):
        server_pool = logging_pool.pool(test_constants.THREAD_CONCURRENCY)
        handler = grpc.method_handlers_generic_handler(
            "test",
            {
                "UnaryUnary": grpc.unary_unary_rpc_method_handler(
                    _handle_unary_unary
                )
            },
        )
        options = (("grpc.so_reuseport", 1),)
        with bound_socket() as (host, port):
            addr = "{}:{}".format(host, port)
            server = grpc.server(server_pool, (handler,), options=options)
            server.add_insecure_port(addr)
            server.start()
        channel = grpc.insecure_channel(addr)
        multi_callable = channel.unary_unary(_UNARY_UNARY)
        self.assertEqual(_RESPONSE, multi_callable(_REQUEST))
        server.stop(None)
        # By default, the channel connectivity is checked every 5s
        # GRPC_CLIENT_CHANNEL_BACKUP_POLL_INTERVAL_MS can be set to change
        # this.
        time.sleep(5.1)
        server = grpc.server(server_pool, (handler,), options=options)
        server.add_insecure_port(addr)
        server.start()
        self.assertEqual(_RESPONSE, multi_callable(_REQUEST))
        server.stop(None)
        channel.close()


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



#!/usr/bin/env python
# Copyright 2015 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import base64
import copy
import os
import random
import subprocess
import sys
import tempfile


def generate(token, s, out_fn):
  with tempfile.NamedTemporaryFile() as der_tmpfile:
    with tempfile.NamedTemporaryFile() as conf_tempfile:
      conf_tempfile.write(str(s))
      conf_tempfile.flush()
      description_tmpfile = tempfile.NamedTemporaryFile()
      subprocess.check_call(["openssl", "asn1parse", "-genconf",
                             conf_tempfile.name, "-i", "-out",
                             der_tmpfile.name],
                            stdout=description_tmpfile)

    with open(out_fn, "w") as output_file:
      description_tmpfile.seek(0)
      output_file.write(description_tmpfile.read())
      output_file.write("-----BEGIN %s-----\n" % token)
      output_file.write(base64.encodestring(der_tmpfile.read()))
      output_file.write("-----END %s-----\n" % token)


class CertificatePoliciesGenerator:
  def __init__(self):
    self.policies = []

  def generate(self, out_fn):
    generate("CERTIFICATE POLICIES", self, out_fn)

  def add_policy(self, policy):
    self.policies.append(policy)

  def __str__(self):
    s = "asn1 = SEQUENCE:certificatePoliciesSequence\n"
    s += "[certificatePoliciesSequence]\n"
    s_suffix = ""
    for n, policy in enumerate(self.policies):
      n1, n2 = (str(policy) + "\n").split("\n", 1)
      if n2:
        s_suffix += n2 + "\n"
      s += "%s%s\n" % (n, n1)

    return s + s_suffix


def policy_qualifier(qualifier_id, qualifier):
  i = random.randint(0, sys.maxint)
  s = "asn1 = SEQUENCE:PolicyQualifierInfoSequence%i\n" % i
  s += "[PolicyQualifierInfoSequence%i]\n" % i
  s += "policyQualifierId = %s\n" % qualifier_id
  s += qualifier
  return s


def cps_uri_qualifier(url):
  return policy_qualifier("OID:id-qt-cps", "cPSUri = IA5STRING:%s\n" % url)


def policy_information(policy_id, qualifiers):
  i = random.randint(0, sys.maxint)
  s = "policyInformation = SEQUENCE:PolicyInformationSequence%i\n" % i
  s += "[PolicyInformationSequence%i]\n" % i
  s += "policyIdentifier = OID:%s\n" % policy_id
  s_suffix = ""
  if qualifiers is not None:
    s += "policyQualifiers = SEQUENCE:PolicyQualifiersSequence%i\n" % i
    s += "[PolicyQualifiersSequence%i]\n" % i
    for n, qualifier in enumerate(qualifiers):
      n1, n2 = (str(qualifier) + "\n").split("\n", 1)
      if n2:
        s_suffix += n2 + "\n"
      s += "%s%s\n" % (n, n1)

  return s + s_suffix


def main():
  p = CertificatePoliciesGenerator()
  p.generate("invalid-empty.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("anyPolicy", None))
  p.generate("anypolicy.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("anyPolicy", [
      cps_uri_qualifier("https://example.com/1_2_3")]))
  p.generate("anypolicy_with_qualifier.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("anyPolicy", [
      policy_qualifier("OID:1.2.3.4", 'foo = UTF8:"hi"')]))
  p.generate("invalid-anypolicy_with_custom_qualifier.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("1.2.3", None))
  p.generate("policy_1_2_3.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("1.2.3", [
      cps_uri_qualifier("https://example.com/1_2_3")]))
  p.generate("policy_1_2_3_with_qualifier.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("1.2.3", [
      policy_qualifier("OID:1.2.3.4", 'foo = UTF8:"hi"')]))
  p.generate("policy_1_2_3_with_custom_qualifier.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("1.2.3", None))
  p.add_policy(policy_information("1.2.3", [
      cps_uri_qualifier("https://example.com/1_2_3")]))
  p.generate("invalid-policy_1_2_3_dupe.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("1.2.3", []))
  p.generate("invalid-policy_1_2_3_with_empty_qualifiers_sequence.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("1.2.3", None))
  p.add_policy(policy_information("1.2.4", None))
  p.generate("policy_1_2_3_and_1_2_4.pem")

  p = CertificatePoliciesGenerator()
  p.add_policy(policy_information("1.2.3", [
      cps_uri_qualifier("https://example.com/1_2_3")]))
  p.add_policy(policy_information("1.2.4", [
      cps_uri_qualifier("http://example.com/1_2_4")]))
  p.generate("policy_1_2_3_and_1_2_4_with_qualifiers.pem")

  generate("CERTIFICATE POLICIES",
           "asn1 = SEQUENCE:certificatePoliciesSequence\n"
           "[certificatePoliciesSequence]\n"
           "policyInformation = SEQUENCE:PolicyInformationSequence\n"
           'extradata = IA5STRING:"unconsumed data"\n'
           "[PolicyInformationSequence]\n"
           "policyIdentifier = OID:1.2.3\n",
           "invalid-policy_1_2_3_policyinformation_unconsumed_data.pem")

  generate("CERTIFICATE POLICIES",
           "asn1 = SEQUENCE:certificatePoliciesSequence\n"
           "[certificatePoliciesSequence]\n"
           "policyInformation = SEQUENCE:PolicyInformationSequence\n"
           "[PolicyInformationSequence]\n"
           "policyIdentifier = OID:1.2.3\n"
           "policyQualifiers = SEQUENCE:PolicyQualifiersSequence\n"
           "[PolicyQualifiersSequence]\n"
           "policyQualifierInfo = SEQUENCE:PolicyQualifierInfoSequence\n"
           "[PolicyQualifierInfoSequence]\n"
           "policyQualifierId = OID:id-qt-cps\n"
           "cPSUri = IA5STRING:https://example.com/1_2_3\n"
           'extradata = IA5STRING:"unconsumed data"\n',
           "invalid-policy_1_2_3_policyqualifierinfo_unconsumed_data.pem")

  generate("CERTIFICATE POLICIES",
           "asn1 = SEQUENCE:certificatePoliciesSequence\n"
           "[certificatePoliciesSequence]\n"
           "policyInformation = SEQUENCE:PolicyInformationSequence\n"
           "[PolicyInformationSequence]\n"
           'policyIdentifier = IA5STRING:"1.2.3"\n',
           "invalid-policy_identifier_not_oid.pem")


if __name__ == "__main__":
  main()




import cython

from .._cache cimport DNSCache
from .._dns cimport DNSAddress, DNSNsec, DNSPointer, DNSRecord, DNSService, DNSText
from .._protocol.outgoing cimport DNSOutgoing
from .._updates cimport RecordUpdateListener
from .._utils.time cimport current_time_millis


cdef object _resolve_all_futures_to_none
cdef object _cached_ip_addresses_wrapper

cdef object _TYPE_SRV
cdef object _TYPE_TXT
cdef object _TYPE_A
cdef object _TYPE_AAAA
cdef object _TYPE_PTR
cdef object _TYPE_NSEC
cdef object _CLASS_IN
cdef object _FLAGS_QR_QUERY

cdef object service_type_name

cdef object DNS_QUESTION_TYPE_QU
cdef object DNS_QUESTION_TYPE_QM

cdef object _IPVersion_All_value
cdef object _IPVersion_V4Only_value

cdef cython.set _ADDRESS_RECORD_TYPES

cdef object TYPE_CHECKING

cdef class ServiceInfo(RecordUpdateListener):

    cdef public cython.bytes text
    cdef public str type
    cdef str _name
    cdef public str key
    cdef public cython.list _ipv4_addresses
    cdef public cython.list _ipv6_addresses
    cdef public object port
    cdef public object weight
    cdef public object priority
    cdef public str server
    cdef public str server_key
    cdef public cython.dict _properties
    cdef public object host_ttl
    cdef public object other_ttl
    cdef public object interface_index
    cdef public cython.set _new_records_futures
    cdef public DNSPointer _dns_pointer_cache
    cdef public DNSService _dns_service_cache
    cdef public DNSText _dns_text_cache
    cdef public cython.list _dns_address_cache
    cdef public cython.set _get_address_and_nsec_records_cache

    @cython.locals(cache=DNSCache)
    cpdef async_update_records(self, object zc, cython.float now, cython.list records)

    @cython.locals(cache=DNSCache)
    cpdef _load_from_cache(self, object zc, cython.float now)

    cdef _unpack_text_into_properties(self)

    cdef _set_properties(self, cython.dict properties)

    cdef _set_text(self, cython.bytes text)

    @cython.locals(record=DNSAddress)
    cdef _get_ip_addresses_from_cache_lifo(self, object zc, cython.float now, object type)

    @cython.locals(
        dns_service_record=DNSService,
        dns_text_record=DNSText,
        dns_address_record=DNSAddress
    )
    cdef _process_record_threadsafe(self, object zc, DNSRecord record, cython.float now)

    @cython.locals(cache=DNSCache)
    cdef cython.list _get_address_records_from_cache_by_type(self, object zc, object _type)

    cdef _set_ipv4_addresses_from_cache(self, object zc, object now)

    cdef _set_ipv6_addresses_from_cache(self, object zc, object now)

    cdef cython.list _ip_addresses_by_version_value(self, object version_value)

    cpdef addresses_by_version(self, object version)

    cpdef ip_addresses_by_version(self, object version)

    @cython.locals(cacheable=cython.bint)
    cdef cython.list _dns_addresses(self, object override_ttls, object version)

    @cython.locals(cacheable=cython.bint)
    cdef DNSPointer _dns_pointer(self, object override_ttl)

    @cython.locals(cacheable=cython.bint)
    cdef DNSService _dns_service(self, object override_ttl)

    @cython.locals(cacheable=cython.bint)
    cdef DNSText _dns_text(self, object override_ttl)

    cdef DNSNsec _dns_nsec(self, cython.list missing_types, object override_ttl)

    @cython.locals(cacheable=cython.bint)
    cdef cython.set _get_address_and_nsec_records(self, object override_ttl)

    cpdef async_clear_cache(self)



# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
from typing import Tuple

from absl import flags
from absl.testing import absltest
import grpc

from framework import xds_url_map_testcase
from framework.helpers import skips
from framework.test_app import client_app

# Type aliases
HostRule = xds_url_map_testcase.HostRule
PathMatcher = xds_url_map_testcase.PathMatcher
GcpResourceManager = xds_url_map_testcase.GcpResourceManager
DumpedXdsConfig = xds_url_map_testcase.DumpedXdsConfig
RpcTypeUnaryCall = xds_url_map_testcase.RpcTypeUnaryCall
XdsTestClient = client_app.XdsTestClient
ExpectedResult = xds_url_map_testcase.ExpectedResult
_Lang = skips.Lang

logger = logging.getLogger(__name__)
flags.adopt_module_key_flags(xds_url_map_testcase)

# The first batch of RPCs don't count towards the result of test case. They are
# meant to prove the communication between driver and client is fine.
_NUM_RPCS = 10
_LENGTH_OF_RPC_SENDING_SEC = 16
# We are using sleep to synchronize test driver and the client... Even though
# the client is sending at QPS rate, we can't assert that exactly QPS *
# SLEEP_DURATION number of RPC is finished. The final completed RPC might be
# slightly more or less.
_NON_RANDOM_ERROR_TOLERANCE = 0.01
_RPC_BEHAVIOR_HEADER_NAME = "rpc-behavior"


def _build_retry_route_rule(retryConditions, num_retries):
    return {
        "priority": 0,
        "matchRules": [
            {"fullPathMatch": "/grpc.testing.TestService/UnaryCall"}
        ],
        "service": GcpResourceManager().default_backend_service(),
        "routeAction": {
            "retryPolicy": {
                "retryConditions": retryConditions,
                "numRetries": num_retries,
            }
        },
    }


def _is_supported(config: skips.TestConfig) -> bool:
    # Per "Retry" in
    # https://github.com/grpc/grpc/blob/master/doc/grpc_xds_features.md
    if config.client_lang in _Lang.CPP | _Lang.JAVA | _Lang.PYTHON:
        return config.version_gte("v1.40.x")
    elif config.client_lang == _Lang.GO:
        return config.version_gte("v1.41.x")
    elif config.client_lang == _Lang.NODE:
        return config.version_gte("v1.8.x")
    return True


class TestRetryUpTo3AttemptsAndFail(xds_url_map_testcase.XdsUrlMapTestCase):
    @staticmethod
    def is_supported(config: skips.TestConfig) -> bool:
        return _is_supported(config)

    @staticmethod
    def url_map_change(
        host_rule: HostRule, path_matcher: PathMatcher
    ) -> Tuple[HostRule, PathMatcher]:
        path_matcher["routeRules"] = [
            _build_retry_route_rule(
                retryConditions=["unavailable"], num_retries=3
            )
        ]
        return host_rule, path_matcher

    def xds_config_validate(self, xds_config: DumpedXdsConfig):
        self.assertNumEndpoints(xds_config, 1)
        retry_config = xds_config.rds["virtualHosts"][0]["routes"][0]["route"][
            "retryPolicy"
        ]
        self.assertEqual(3, retry_config["numRetries"])
        self.assertEqual("unavailable", retry_config["retryOn"])

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        self.configure_and_send(
            test_client,
            rpc_types=(RpcTypeUnaryCall,),
            metadata=[
                (
                    RpcTypeUnaryCall,
                    _RPC_BEHAVIOR_HEADER_NAME,
                    "succeed-on-retry-attempt-4,error-code-14",
                )
            ],
            num_rpcs=_NUM_RPCS,
        )
        self.assertRpcStatusCode(
            test_client,
            expected=(
                ExpectedResult(
                    rpc_type=RpcTypeUnaryCall,
                    status_code=grpc.StatusCode.UNAVAILABLE,
                    ratio=1,
                ),
            ),
            length=_LENGTH_OF_RPC_SENDING_SEC,
            tolerance=_NON_RANDOM_ERROR_TOLERANCE,
        )


class TestRetryUpTo4AttemptsAndSucceed(xds_url_map_testcase.XdsUrlMapTestCase):
    @staticmethod
    def is_supported(config: skips.TestConfig) -> bool:
        return _is_supported(config)

    @staticmethod
    def url_map_change(
        host_rule: HostRule, path_matcher: PathMatcher
    ) -> Tuple[HostRule, PathMatcher]:
        path_matcher["routeRules"] = [
            _build_retry_route_rule(
                retryConditions=["unavailable"], num_retries=4
            )
        ]
        return host_rule, path_matcher

    def xds_config_validate(self, xds_config: DumpedXdsConfig):
        self.assertNumEndpoints(xds_config, 1)
        retry_config = xds_config.rds["virtualHosts"][0]["routes"][0]["route"][
            "retryPolicy"
        ]
        self.assertEqual(4, retry_config["numRetries"])
        self.assertEqual("unavailable", retry_config["retryOn"])

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        self.configure_and_send(
            test_client,
            rpc_types=(RpcTypeUnaryCall,),
            metadata=[
                (
                    RpcTypeUnaryCall,
                    _RPC_BEHAVIOR_HEADER_NAME,
                    "succeed-on-retry-attempt-4,error-code-14",
                )
            ],
            num_rpcs=_NUM_RPCS,
        )
        self.assertRpcStatusCode(
            test_client,
            expected=(
                ExpectedResult(
                    rpc_type=RpcTypeUnaryCall,
                    status_code=grpc.StatusCode.OK,
                    ratio=1,
                ),
            ),
            length=_LENGTH_OF_RPC_SENDING_SEC,
            tolerance=_NON_RANDOM_ERROR_TOLERANCE,
        )


if __name__ == "__main__":
    absltest.main()



#!/usr/bin/env python
# Copyright 2015 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""This script is called without any arguments to re-format all of the *.pem
files in the script's parent directory.

The main formatting change is to run "openssl asn1parse" for each of the PEM
block sections (except for DATA), and add that output to the comment.

Refer to the README file for more information.
"""

import glob
import os
import re
import base64
import subprocess


def Transform(file_data):
  """Returns a transformed (formatted) version of file_data"""

  result = ''

  for block in GetPemBlocks(file_data):
    if len(result) != 0:
      result += '\n'

    # If there was a user comment (non-script-generated comment) associated
    # with the block, output it immediately before the block.
    user_comment = GetUserComment(block.comment)
    if user_comment:
      result += user_comment

    # For every block except for DATA, try to pretty print the parsed ASN.1.
    # DATA blocks likely would be DER in practice, but for the purposes of
    # these tests seeing its structure doesn't clarify
    # anything and is just a distraction.
    if block.name != 'DATA':
      generated_comment = GenerateCommentForBlock(block.name, block.data)
      result += generated_comment + '\n'


    result += MakePemBlockString(block.name, block.data)

  return result


def GenerateCommentForBlock(block_name, block_data):
  """Returns a string describing the ASN.1 structure of block_data"""

  p = subprocess.Popen(['openssl', 'asn1parse', '-i', '-inform', 'DER'],
                       stdout=subprocess.PIPE, stdin=subprocess.PIPE,
                       stderr=subprocess.PIPE)
  stdout_data, stderr_data = p.communicate(input=block_data)
  generated_comment = '$ openssl asn1parse -i < [%s]\n%s' % (block_name,
                                                             stdout_data)
  return generated_comment.strip('\n')



def GetUserComment(comment):
  """Removes any script-generated lines (everything after the $ openssl line)"""

  # Consider everything after "$ openssl" to be a generated comment.
  comment = comment.split('$ openssl asn1parse -i', 1)[0]
  if IsEntirelyWhiteSpace(comment):
    comment = ''
  return comment


def MakePemBlockString(name, data):
  return ('-----BEGIN %s-----\n'
          '%s'
          '-----END %s-----\n') % (name, EncodeDataForPem(data), name)


def GetPemFilePaths():
  """Returns an iterable for all the paths to the PEM test files"""

  base_dir = os.path.dirname(os.path.realpath(__file__))
  return glob.iglob(os.path.join(base_dir, '*.pem'))


def ReadFileToString(path):
  with open(path, 'r') as f:
    return f.read()


def WrapTextToLineWidth(text, column_width):
  result = ''
  pos = 0
  while pos < len(text):
    result += text[pos : pos + column_width] + '\n'
    pos += column_width
  return result


def EncodeDataForPem(data):
  result = base64.b64encode(data)
  return WrapTextToLineWidth(result, 75)


class PemBlock(object):
  def __init__(self):
    self.name = None
    self.data = None
    self.comment = None


def StripAllWhitespace(text):
  pattern = re.compile(r'\s+')
  return re.sub(pattern, '', text)


def IsEntirelyWhiteSpace(text):
  return len(StripAllWhitespace(text)) == 0


def DecodePemBlockData(text):
  text = StripAllWhitespace(text)
  return base64.b64decode(text)


def GetPemBlocks(data):
  """Returns an iterable of PemBlock"""

  comment_start = 0

  regex = re.compile(r'-----BEGIN ([\w ]+)-----(.*?)-----END \1-----',
                     re.DOTALL)

  for match in regex.finditer(data):
    block = PemBlock()

    block.name = match.group(1)
    block.data = DecodePemBlockData(match.group(2))

    # Keep track of any non-PEM text above blocks
    block.comment = data[comment_start : match.start()].strip()
    comment_start = match.end()

    yield block


def WriteStringToFile(data, path):
  with open(path, "w") as f:
    f.write(data)


def main():
  for path in GetPemFilePaths():
    print "Processing %s ..." % (path)
    original_data = ReadFileToString(path)
    transformed_data = Transform(original_data)
    if original_data != transformed_data:
      WriteStringToFile(transformed_data, path)
      print "Rewrote %s" % (path)


if __name__ == "__main__":
  main()




import cython

from .._cache cimport DNSCache
from .._dns cimport DNSRecord
from .._protocol.incoming cimport DNSIncoming


cdef cython.float _DNS_PTR_MIN_TTL
cdef object _ADDRESS_RECORD_TYPES
cdef object RecordUpdate
cdef object TYPE_CHECKING
cdef object _TYPE_PTR


cdef class RecordManager:

    cdef public object zc
    cdef public DNSCache cache
    cdef public cython.set listeners

    cpdef async_updates(self, object now, object records)

    cpdef async_updates_complete(self, object notify)

    @cython.locals(
        cache=DNSCache,
        record=DNSRecord,
        maybe_entry=DNSRecord,
        now_float=cython.float
    )
    cpdef async_updates_from_response(self, DNSIncoming msg)

    cpdef async_add_listener(self, object listener, object question)

    cpdef async_remove_listener(self, object listener)



# Copyright 2019 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests behavior of the grpc.aio.Channel class."""

import logging
import os
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit.framework.common import test_constants
from tests_aio.unit._constants import UNARY_CALL_WITH_SLEEP_VALUE
from tests_aio.unit._constants import UNREACHABLE_TARGET
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_UNARY_CALL_METHOD = "/grpc.testing.TestService/UnaryCall"
_UNARY_CALL_METHOD_WITH_SLEEP = "/grpc.testing.TestService/UnaryCallWithSleep"
_STREAMING_OUTPUT_CALL_METHOD = "/grpc.testing.TestService/StreamingOutputCall"

_INVOCATION_METADATA = (
    ("x-grpc-test-echo-initial", "initial-md-value"),
    ("x-grpc-test-echo-trailing-bin", b"\x00\x02"),
)

_NUM_STREAM_RESPONSES = 5
_REQUEST_PAYLOAD_SIZE = 7
_RESPONSE_PAYLOAD_SIZE = 42


class TestChannel(AioTestBase):
    async def setUp(self):
        self._server_target, self._server = await start_test_server()

    async def tearDown(self):
        await self._server.stop(None)

    async def test_async_context(self):
        async with aio.insecure_channel(self._server_target) as channel:
            hi = channel.unary_unary(
                _UNARY_CALL_METHOD,
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            await hi(messages_pb2.SimpleRequest())

    async def test_unary_unary(self):
        async with aio.insecure_channel(self._server_target) as channel:
            hi = channel.unary_unary(
                _UNARY_CALL_METHOD,
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            response = await hi(messages_pb2.SimpleRequest())

            self.assertIsInstance(response, messages_pb2.SimpleResponse)

    async def test_unary_call_times_out(self):
        async with aio.insecure_channel(self._server_target) as channel:
            hi = channel.unary_unary(
                _UNARY_CALL_METHOD_WITH_SLEEP,
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            with self.assertRaises(grpc.RpcError) as exception_context:
                await hi(
                    messages_pb2.SimpleRequest(),
                    timeout=UNARY_CALL_WITH_SLEEP_VALUE / 2,
                )

            (
                _,
                details,
            ) = (
                grpc.StatusCode.DEADLINE_EXCEEDED.value
            )  # pylint: disable=unused-variable
            self.assertEqual(
                grpc.StatusCode.DEADLINE_EXCEEDED,
                exception_context.exception.code(),
            )
            self.assertEqual(
                details.title(), exception_context.exception.details()
            )
            self.assertIsNotNone(exception_context.exception.initial_metadata())
            self.assertIsNotNone(
                exception_context.exception.trailing_metadata()
            )

    @unittest.skipIf(
        os.name == "nt", "TODO: https://github.com/grpc/grpc/issues/21658"
    )
    async def test_unary_call_does_not_times_out(self):
        async with aio.insecure_channel(self._server_target) as channel:
            hi = channel.unary_unary(
                _UNARY_CALL_METHOD_WITH_SLEEP,
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            call = hi(
                messages_pb2.SimpleRequest(),
                timeout=UNARY_CALL_WITH_SLEEP_VALUE * 5,
            )
            self.assertEqual(await call.code(), grpc.StatusCode.OK)

    async def test_unary_stream(self):
        channel = aio.insecure_channel(self._server_target)
        stub = test_pb2_grpc.TestServiceStub(channel)

        # Prepares the request
        request = messages_pb2.StreamingOutputCallRequest()
        for _ in range(_NUM_STREAM_RESPONSES):
            request.response_parameters.append(
                messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
            )

        # Invokes the actual RPC
        call = stub.StreamingOutputCall(request)

        # Validates the responses
        response_cnt = 0
        async for response in call:
            response_cnt += 1
            self.assertIs(
                type(response), messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        self.assertEqual(_NUM_STREAM_RESPONSES, response_cnt)
        self.assertEqual(await call.code(), grpc.StatusCode.OK)
        await channel.close()

    async def test_stream_unary_using_write(self):
        channel = aio.insecure_channel(self._server_target)
        stub = test_pb2_grpc.TestServiceStub(channel)

        # Invokes the actual RPC
        call = stub.StreamingInputCall()

        # Prepares the request
        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        # Sends out requests
        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(request)
        await call.done_writing()

        # Validates the responses
        response = await call
        self.assertIsInstance(response, messages_pb2.StreamingInputCallResponse)
        self.assertEqual(
            _NUM_STREAM_RESPONSES * _REQUEST_PAYLOAD_SIZE,
            response.aggregated_payload_size,
        )

        self.assertEqual(await call.code(), grpc.StatusCode.OK)
        await channel.close()

    async def test_stream_unary_using_async_gen(self):
        channel = aio.insecure_channel(self._server_target)
        stub = test_pb2_grpc.TestServiceStub(channel)

        # Prepares the request
        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        async def gen():
            for _ in range(_NUM_STREAM_RESPONSES):
                yield request

        # Invokes the actual RPC
        call = stub.StreamingInputCall(gen())

        # Validates the responses
        response = await call
        self.assertIsInstance(response, messages_pb2.StreamingInputCallResponse)
        self.assertEqual(
            _NUM_STREAM_RESPONSES * _REQUEST_PAYLOAD_SIZE,
            response.aggregated_payload_size,
        )

        self.assertEqual(await call.code(), grpc.StatusCode.OK)
        await channel.close()

    async def test_stream_stream_using_read_write(self):
        channel = aio.insecure_channel(self._server_target)
        stub = test_pb2_grpc.TestServiceStub(channel)

        # Invokes the actual RPC
        call = stub.FullDuplexCall()

        # Prepares the request
        request = messages_pb2.StreamingOutputCallRequest()
        request.response_parameters.append(
            messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
        )

        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(request)
            response = await call.read()
            self.assertIsInstance(
                response, messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        await call.done_writing()

        self.assertEqual(grpc.StatusCode.OK, await call.code())
        await channel.close()

    async def test_stream_stream_using_async_gen(self):
        channel = aio.insecure_channel(self._server_target)
        stub = test_pb2_grpc.TestServiceStub(channel)

        # Prepares the request
        request = messages_pb2.StreamingOutputCallRequest()
        request.response_parameters.append(
            messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
        )

        async def gen():
            for _ in range(_NUM_STREAM_RESPONSES):
                yield request

        # Invokes the actual RPC
        call = stub.FullDuplexCall(gen())

        async for response in call:
            self.assertIsInstance(
                response, messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        self.assertEqual(grpc.StatusCode.OK, await call.code())
        await channel.close()


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The module contains helpers to initialize and configure logging."""
import functools
import pathlib

from absl import flags
from absl import logging


def _ensure_flags_parsed() -> None:
    if not flags.FLAGS.is_parsed():
        raise flags.UnparsedFlagAccessError("Must initialize absl flags first.")


@functools.lru_cache(None)
def log_get_root_dir() -> pathlib.Path:
    _ensure_flags_parsed()
    log_root = pathlib.Path(logging.find_log_dir()).absolute()
    logging.info("Log root dir: %s", log_root)
    return log_root


def log_dir_mkdir(name: str) -> pathlib.Path:
    """Creates and returns a subdir with the given name in the log folder."""
    if len(pathlib.Path(name).parts) != 1:
        raise ValueError(f"Dir name must be a single component; got: {name}")
    if ".." in name:
        raise ValueError(f"Dir name must not be above the log root.")
    log_subdir = log_get_root_dir() / name
    if log_subdir.exists() and log_subdir.is_dir():
        logging.debug("Using existing log subdir: %s", log_subdir)
    else:
        log_subdir.mkdir()
        logging.debug("Created log subdir: %s", log_subdir)

    return log_subdir



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
from typing import Optional

from framework.infrastructure import k8s
import framework.infrastructure.traffic_director_gamma as td_gamma
from framework.test_app import client_app
from framework.test_app import server_app
from framework.test_app.runners.k8s import gamma_server_runner
from framework.test_app.runners.k8s import k8s_xds_client_runner
import framework.xds_k8s_testcase as xds_k8s_testcase

GammaServerRunner = gamma_server_runner.GammaServerRunner
KubernetesClientRunner = k8s_xds_client_runner.KubernetesClientRunner
XdsTestClient = client_app.XdsTestClient
XdsTestServer = server_app.XdsTestServer

logger = logging.getLogger(__name__)

# We never actually hit this timeout under normal circumstances, so this large
# value is acceptable.
_TERMINATION_GRACE_PERIOD_SECONDS = 600


# TODO(sergiitk): [GAMMA] Move into framework/test_cases
class GammaXdsKubernetesTestCase(xds_k8s_testcase.RegularXdsKubernetesTestCase):
    server_runner: GammaServerRunner
    frontend_service_name: str
    pre_stop_hook: Optional[bool] = None

    def setUp(self):
        """Hook method for setting up the test fixture before exercising it."""
        # TODO(sergiitk): [GAMMA] Remove when refactored to be TD-manager-less.
        # pylint: disable=bad-super-call
        # Skips RegularXdsKubernetesTestCase and IsolatedXdsKubernetesTestCase
        # and calls setUp on XdsKubernetesBaseTestCase.
        # IsolatedXdsKubernetesTestCase randomizes server_xds_port when it's 0,
        # and in GAMMA we always need it unset.
        # Calls XdsKubernetesBaseTestCase.setUp():
        super(xds_k8s_testcase.IsolatedXdsKubernetesTestCase, self).setUp()
        # pylint: enable=bad-super-call

        if self.pre_stop_hook is None:
            self.pre_stop_hook = False

        # Random suffix per test.
        self.createRandomSuffix()

        # TODO(sergiitk): [GAMMA] Make a TD-manager-less base test case
        # TD Manager
        self.td = self.initTrafficDirectorManager()

        # Generate unique mesh name too.
        self.frontend_service_name = (
            f"{self.resource_prefix}-{self.resource_suffix.lower()}"
        )

        # Test Server runner
        self.server_namespace = GammaServerRunner.make_namespace_name(
            self.resource_prefix, self.resource_suffix
        )
        self.server_runner = self.initKubernetesServerRunner()

        # Test Client runner
        self.client_namespace = KubernetesClientRunner.make_namespace_name(
            self.resource_prefix, self.resource_suffix
        )
        self.client_runner = self.initKubernetesClientRunner()

        # Cleanup.
        self.force_cleanup = True
        self.force_cleanup_namespace = True

    # TODO(sergiitk): [GAMMA] Make a TD-manager-less base test case
    def initTrafficDirectorManager(
        self,
    ) -> td_gamma.TrafficDirectorGammaManager:
        return td_gamma.TrafficDirectorGammaManager(
            self.gcp_api_manager,
            project=self.project,
            resource_prefix=self.resource_prefix,
            resource_suffix=self.resource_suffix,
            network=self.network,
            compute_api_version=self.compute_api_version,
        )

    def initKubernetesServerRunner(self) -> GammaServerRunner:
        return GammaServerRunner(
            k8s.KubernetesNamespace(
                self.k8s_api_manager, self.server_namespace
            ),
            self.frontend_service_name,
            deployment_name=self.server_name,
            image_name=self.server_image,
            td_bootstrap_image=self.td_bootstrap_image,
            gcp_project=self.project,
            gcp_api_manager=self.gcp_api_manager,
            gcp_service_account=self.gcp_service_account,
            xds_server_uri=self.xds_server_uri,
            network=self.network,
            debug_use_port_forwarding=self.debug_use_port_forwarding,
            enable_workload_identity=self.enable_workload_identity,
            termination_grace_period_seconds=_TERMINATION_GRACE_PERIOD_SECONDS,
            pre_stop_hook=self.pre_stop_hook,
        )

    def startTestClient(
        self, test_server: XdsTestServer, **kwargs
    ) -> XdsTestClient:
        server_target = (
            f"xds:///{self.frontend_service_name}"
            f".{self.server_namespace}.svc.cluster.local"
            f":{test_server.rpc_port}"
        )
        kwargs.setdefault("generate_mesh_id", True)
        return self._start_test_client(server_target=server_target, **kwargs)



#!/usr/bin/env python


""" Unit tests for zeroconf._cache. """


import logging
import unittest
import unittest.mock

import zeroconf as r
from zeroconf import const

log = logging.getLogger('zeroconf')
original_logging_level = logging.NOTSET


def setup_module():
    global original_logging_level
    original_logging_level = log.level
    log.setLevel(logging.DEBUG)


def teardown_module():
    if original_logging_level != logging.NOTSET:
        log.setLevel(original_logging_level)


class TestDNSCache(unittest.TestCase):
    def test_order(self):
        record1 = r.DNSAddress('a', const._TYPE_SOA, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_SOA, const._CLASS_IN, 1, b'b')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        entry = r.DNSEntry('a', const._TYPE_SOA, const._CLASS_IN)
        cached_record = cache.get(entry)
        assert cached_record == record2

    def test_adding_same_record_to_cache_different_ttls_with_get(self):
        """We should always get back the last entry we added if there are different TTLs.

        This ensures we only have one source of truth for TTLs as a record cannot
        be both expired and not expired.
        """
        record1 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 10, b'a')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        entry = r.DNSEntry(record2.name, const._TYPE_A, const._CLASS_IN)
        cached_record = cache.get(entry)
        assert cached_record == record2

    def test_adding_same_record_to_cache_different_ttls_with_get_all(self):
        """Verify we only get one record back.

        The last record added should replace the previous since two
        records with different ttls are __eq__. This ensures we
        only have one source of truth for TTLs as a record cannot
        be both expired and not expired.
        """
        record1 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 10, b'a')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        cached_records = cache.get_all_by_details('a', const._TYPE_A, const._CLASS_IN)
        assert cached_records == [record2]

    def test_cache_empty_does_not_leak_memory_by_leaving_empty_list(self):
        record1 = r.DNSAddress('a', const._TYPE_SOA, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_SOA, const._CLASS_IN, 1, b'b')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert 'a' in cache.cache
        cache.async_remove_records([record1, record2])
        assert 'a' not in cache.cache

    def test_cache_empty_multiple_calls(self):
        record1 = r.DNSAddress('a', const._TYPE_SOA, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_SOA, const._CLASS_IN, 1, b'b')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert 'a' in cache.cache
        cache.async_remove_records([record1, record2])
        assert 'a' not in cache.cache


class TestDNSAsyncCacheAPI(unittest.TestCase):
    def test_async_get_unique(self):
        record1 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'b')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert cache.async_get_unique(record1) == record1
        assert cache.async_get_unique(record2) == record2

    def test_async_all_by_details(self):
        record1 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'b')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert set(cache.async_all_by_details('a', const._TYPE_A, const._CLASS_IN)) == {record1, record2}

    def test_async_entries_with_server(self):
        record1 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 85, 'ab'
        )
        record2 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 80, 'ab'
        )
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert set(cache.async_entries_with_server('ab')) == {record1, record2}
        assert set(cache.async_entries_with_server('AB')) == {record1, record2}

    def test_async_entries_with_name(self):
        record1 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 85, 'ab'
        )
        record2 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 80, 'ab'
        )
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert set(cache.async_entries_with_name('irrelevant')) == {record1, record2}
        assert set(cache.async_entries_with_name('Irrelevant')) == {record1, record2}


# These functions have been seen in other projects so
# we try to maintain a stable API for all the threadsafe getters
class TestDNSCacheAPI(unittest.TestCase):
    def test_get(self):
        record1 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'b')
        record3 = r.DNSAddress('a', const._TYPE_AAAA, const._CLASS_IN, 1, b'ipv6')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2, record3])
        assert cache.get(record1) == record1
        assert cache.get(record2) == record2
        assert cache.get(r.DNSEntry('a', const._TYPE_A, const._CLASS_IN)) == record2
        assert cache.get(r.DNSEntry('a', const._TYPE_AAAA, const._CLASS_IN)) == record3
        assert cache.get(r.DNSEntry('notthere', const._TYPE_A, const._CLASS_IN)) is None

    def test_get_by_details(self):
        record1 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'b')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert cache.get_by_details('a', const._TYPE_A, const._CLASS_IN) == record2

    def test_get_all_by_details(self):
        record1 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'a')
        record2 = r.DNSAddress('a', const._TYPE_A, const._CLASS_IN, 1, b'b')
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert set(cache.get_all_by_details('a', const._TYPE_A, const._CLASS_IN)) == {record1, record2}

    def test_entries_with_server(self):
        record1 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 85, 'ab'
        )
        record2 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 80, 'ab'
        )
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert set(cache.entries_with_server('ab')) == {record1, record2}
        assert set(cache.entries_with_server('AB')) == {record1, record2}

    def test_entries_with_name(self):
        record1 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 85, 'ab'
        )
        record2 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 80, 'ab'
        )
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert set(cache.entries_with_name('irrelevant')) == {record1, record2}
        assert set(cache.entries_with_name('Irrelevant')) == {record1, record2}

    def test_current_entry_with_name_and_alias(self):
        record1 = r.DNSPointer(
            'irrelevant', const._TYPE_PTR, const._CLASS_IN, const._DNS_OTHER_TTL, 'x.irrelevant'
        )
        record2 = r.DNSPointer(
            'irrelevant', const._TYPE_PTR, const._CLASS_IN, const._DNS_OTHER_TTL, 'y.irrelevant'
        )
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert cache.current_entry_with_name_and_alias('irrelevant', 'x.irrelevant') == record1

    def test_name(self):
        record1 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 85, 'ab'
        )
        record2 = r.DNSService(
            'irrelevant', const._TYPE_SRV, const._CLASS_IN, const._DNS_HOST_TTL, 0, 0, 80, 'ab'
        )
        cache = r.DNSCache()
        cache.async_add_records([record1, record2])
        assert cache.names() == ['irrelevant']



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Common functionality for running xDS Test Client and Server remotely.
"""
from abc import ABCMeta
from abc import abstractmethod
import functools
import pathlib
import threading
from typing import Dict, Optional
import urllib.parse

from absl import flags

from framework import xds_flags
from framework.helpers import logs

flags.adopt_module_key_flags(logs)
_LOGS_SUBDIR = "test_app_logs"


class RunnerError(Exception):
    """Error running xDS Test App running remotely."""


class BaseRunner(metaclass=ABCMeta):
    _logs_subdir: Optional[pathlib.Path] = None
    _log_stop_event: Optional[threading.Event] = None

    def __init__(self):
        if xds_flags.COLLECT_APP_LOGS.value:
            self._logs_subdir = logs.log_dir_mkdir(_LOGS_SUBDIR)
            self._log_stop_event = threading.Event()

    @property
    @functools.lru_cache(None)
    def should_collect_logs(self) -> bool:
        return self._logs_subdir is not None

    @property
    @functools.lru_cache(None)
    def logs_subdir(self) -> pathlib.Path:
        if not self.should_collect_logs:
            raise FileNotFoundError("Log collection is not enabled.")
        return self._logs_subdir

    @property
    def log_stop_event(self) -> threading.Event:
        if not self.should_collect_logs:
            raise ValueError("Log collection is not enabled.")
        return self._log_stop_event

    def maybe_stop_logging(self):
        if self.should_collect_logs and not self.log_stop_event.is_set():
            self.log_stop_event.set()

    @abstractmethod
    def run(self, **kwargs):
        pass

    @abstractmethod
    def cleanup(self, *, force=False):
        pass

    @classmethod
    def _logs_explorer_link_from_params(
        cls,
        *,
        gcp_ui_url: str,
        gcp_project: str,
        query: Dict[str, str],
        request: Optional[Dict[str, str]] = None,
    ) -> str:
        req_merged = {"query": cls._logs_explorer_query(query)}
        if request is not None:
            req_merged.update(request)

        req = cls._logs_explorer_request(req_merged)
        return f"https://{gcp_ui_url}/logs/query;{req}?project={gcp_project}"

    @classmethod
    def _logs_explorer_query(cls, query: Dict[str, str]) -> str:
        return "\n".join(f'{k}="{v}"' for k, v in query.items())

    @classmethod
    def _logs_explorer_request(cls, req: Dict[str, str]) -> str:
        return ";".join(
            f"{k}={cls._logs_explorer_quote(v)}" for k, v in req.items()
        )

    @classmethod
    def _logs_explorer_quote(cls, value: str) -> str:
        return urllib.parse.quote_plus(value, safe=":")



#!/usr/bin/env python3
#
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Configurable constants for the bm_*.py family """

_AVAILABLE_BENCHMARK_TESTS = [
    "bm_fullstack_unary_ping_pong",
    "bm_fullstack_streaming_ping_pong",
    "bm_fullstack_streaming_pump",
    "bm_closure",
    "bm_cq",
    "bm_call_create",
    "bm_chttp2_hpack",
    "bm_chttp2_transport",
    "bm_pollset",
]

_INTERESTING = (
    "cpu_time",
    "real_time",
    "locks_per_iteration",
    "allocs_per_iteration",
    "writes_per_iteration",
    "atm_cas_per_iteration",
    "atm_add_per_iteration",
    "nows_per_iteration",
    "cli_transport_stalls_per_iteration",
    "cli_stream_stalls_per_iteration",
    "svr_transport_stalls_per_iteration",
    "svr_stream_stalls_per_iteration",
    "http2_pings_sent_per_iteration",
)



""" Multicast DNS Service Discovery for Python, v0.14-wmcbrine
    Copyright 2003 Paul Scott-Murphy, 2014 William McBrine

    This module provides a framework for the use of DNS Service Discovery
    using IP multicast.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
    USA
"""

from operator import attrgetter
from typing import Dict, List, Set

from .._dns import DNSQuestion, DNSRecord
from .._protocol.outgoing import DNSOutgoing
from ..const import _FLAGS_AA, _FLAGS_QR_RESPONSE

_AnswerWithAdditionalsType = Dict[DNSRecord, Set[DNSRecord]]

int_ = int


MULTICAST_DELAY_RANDOM_INTERVAL = (20, 120)

NAME_GETTER = attrgetter('name')

_FLAGS_QR_RESPONSE_AA = _FLAGS_QR_RESPONSE | _FLAGS_AA

float_ = float


class QuestionAnswers:
    """A group of answers to a question."""

    __slots__ = ('ucast', 'mcast_now', 'mcast_aggregate', 'mcast_aggregate_last_second')

    def __init__(
        self,
        ucast: _AnswerWithAdditionalsType,
        mcast_now: _AnswerWithAdditionalsType,
        mcast_aggregate: _AnswerWithAdditionalsType,
        mcast_aggregate_last_second: _AnswerWithAdditionalsType,
    ) -> None:
        """Initialize a QuestionAnswers."""
        self.ucast = ucast
        self.mcast_now = mcast_now
        self.mcast_aggregate = mcast_aggregate
        self.mcast_aggregate_last_second = mcast_aggregate_last_second


class AnswerGroup:
    """A group of answers scheduled to be sent at the same time."""

    __slots__ = ('send_after', 'send_before', 'answers')

    def __init__(self, send_after: float_, send_before: float_, answers: _AnswerWithAdditionalsType) -> None:
        self.send_after = send_after  # Must be sent after this time
        self.send_before = send_before  # Must be sent before this time
        self.answers = answers


def construct_outgoing_multicast_answers(answers: _AnswerWithAdditionalsType) -> DNSOutgoing:
    """Add answers and additionals to a DNSOutgoing."""
    out = DNSOutgoing(_FLAGS_QR_RESPONSE_AA, True)
    _add_answers_additionals(out, answers)
    return out


def construct_outgoing_unicast_answers(
    answers: _AnswerWithAdditionalsType, ucast_source: bool, questions: List[DNSQuestion], id_: int_
) -> DNSOutgoing:
    """Add answers and additionals to a DNSOutgoing."""
    out = DNSOutgoing(_FLAGS_QR_RESPONSE_AA, False, id_)
    # Adding the questions back when the source is legacy unicast behavior
    if ucast_source:
        for question in questions:
            out.add_question(question)
    _add_answers_additionals(out, answers)
    return out


def _add_answers_additionals(out: DNSOutgoing, answers: _AnswerWithAdditionalsType) -> None:
    # Find additionals and suppress any additionals that are already in answers
    sending: Set[DNSRecord] = set(answers)
    # Answers are sorted to group names together to increase the chance
    # that similar names will end up in the same packet and can reduce the
    # overall size of the outgoing response via name compression
    for answer in sorted(answers, key=NAME_GETTER):
        out.add_answer_at_time(answer, 0)
        additionals = answers[answer]
        for additional in additionals:
            if additional not in sending:
                out.add_additional_answer(additional)
                sending.add(additional)



#!/usr/bin/env python
# Copyright 2015 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Certificate chain where the root certificate is not self-signed (or
self-issued for that matter)."""

import sys
sys.path += ['../..']

import gencerts

shadow_root = gencerts.create_self_signed_root_certificate('ShadowRoot')

# Non-self-signed root certificate.
root = gencerts.create_intermediate_certificate('Root', shadow_root)

# Intermediate certificate.
intermediate = gencerts.create_intermediate_certificate('Intermediate', root)

# Target certificate.
target = gencerts.create_end_entity_certificate('Target', intermediate)

chain = [target, intermediate, root]
gencerts.write_chain(__doc__, chain, 'chain.pem')



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests AioRpcError class."""

import logging
import pickle
import unittest

import grpc
from grpc.aio._call import AioRpcError
from grpc.experimental import aio

from tests_aio.unit._test_base import AioTestBase

_TEST_INITIAL_METADATA = aio.Metadata(
    ("initial metadata key", "initial metadata value")
)
_TEST_TRAILING_METADATA = aio.Metadata(
    ("trailing metadata key", "trailing metadata value")
)
_TEST_DEBUG_ERROR_STRING = "{This is a debug string}"


class TestAioRpcError(unittest.TestCase):
    def test_attributes(self):
        aio_rpc_error = AioRpcError(
            grpc.StatusCode.CANCELLED,
            initial_metadata=_TEST_INITIAL_METADATA,
            trailing_metadata=_TEST_TRAILING_METADATA,
            details="details",
            debug_error_string=_TEST_DEBUG_ERROR_STRING,
        )
        self.assertEqual(aio_rpc_error.code(), grpc.StatusCode.CANCELLED)
        self.assertEqual(aio_rpc_error.details(), "details")
        self.assertEqual(
            aio_rpc_error.initial_metadata(), _TEST_INITIAL_METADATA
        )
        self.assertEqual(
            aio_rpc_error.trailing_metadata(), _TEST_TRAILING_METADATA
        )
        self.assertEqual(
            aio_rpc_error.debug_error_string(), _TEST_DEBUG_ERROR_STRING
        )

    def test_pickle(self):
        aio_rpc_error = AioRpcError(
            grpc.StatusCode.CANCELLED,
            initial_metadata=_TEST_INITIAL_METADATA,
            trailing_metadata=_TEST_TRAILING_METADATA,
            details="details",
            debug_error_string=_TEST_DEBUG_ERROR_STRING,
        )
        dump_error = pickle.dumps(aio_rpc_error)
        loaded_error = pickle.loads(dump_error)
        self.assertEqual(loaded_error.code(), grpc.StatusCode.CANCELLED)
        self.assertEqual(loaded_error.details(), "details")
        self.assertEqual(
            loaded_error.initial_metadata(), _TEST_INITIAL_METADATA
        )
        self.assertEqual(
            loaded_error.trailing_metadata(), _TEST_TRAILING_METADATA
        )
        self.assertEqual(
            loaded_error.debug_error_string(), _TEST_DEBUG_ERROR_STRING
        )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
import os
import pathlib
import threading
from typing import Any, Callable, Optional, TextIO

from kubernetes import client
from kubernetes.watch import watch

logger = logging.getLogger(__name__)


class PodLogCollector(threading.Thread):
    """A thread that streams logs from the remote pod to a local file."""

    pod_name: str
    namespace_name: str
    stop_event: threading.Event
    drain_event: threading.Event
    log_path: pathlib.Path
    log_to_stdout: bool
    log_timestamps: bool
    error_backoff_sec: int
    _out_stream: Optional[TextIO]
    _watcher: Optional[watch.Watch]
    _read_pod_log_fn: Callable[..., Any]

    def __init__(
        self,
        *,
        pod_name: str,
        namespace_name: str,
        read_pod_log_fn: Callable[..., Any],
        stop_event: threading.Event,
        log_path: pathlib.Path,
        log_to_stdout: bool = False,
        log_timestamps: bool = False,
        error_backoff_sec: int = 5,
    ):
        self.pod_name = pod_name
        self.namespace_name = namespace_name
        self.stop_event = stop_event
        # Used to indicate log draining happened. Turned out to be not as useful
        # in cases when the logging happens rarely because the blocking happens
        # in the native code, which doesn't yield until the next log message.
        self.drain_event = threading.Event()
        self.log_path = log_path
        self.log_to_stdout = log_to_stdout
        self.log_timestamps = log_timestamps
        self.error_backoff_sec = error_backoff_sec
        self._read_pod_log_fn = read_pod_log_fn
        self._out_stream = None
        self._watcher = None
        super().__init__(name=f"pod-log-{pod_name}", daemon=True)

    def run(self):
        logger.info(
            "Starting log collection thread %i for %s",
            self.ident,
            self.pod_name,
        )
        try:
            self._out_stream = open(
                self.log_path, "w", errors="ignore", encoding="utf-8"
            )
            while not self.stop_event.is_set():
                self._stream_log()
        finally:
            self._stop()

    def flush(self):
        """Flushes the log file buffer. May be called from the main thread."""
        if self._out_stream:
            self._out_stream.flush()
            os.fsync(self._out_stream.fileno())

    def _stop(self):
        if self._watcher is not None:
            self._watcher.stop()
            self._watcher = None
        if self._out_stream is not None:
            self._write(
                f"Finished log collection for pod {self.pod_name}",
                force_flush=True,
            )
            self._out_stream.close()
            self._out_stream = None
        self.drain_event.set()

    def _stream_log(self):
        try:
            self._restart_stream()
        except client.ApiException as e:
            self._write(f"Exception fetching logs: {e}")
            self._write(
                (
                    f"Restarting log fetching in {self.error_backoff_sec} sec. "
                    "Will attempt to read from the beginning, but log "
                    "truncation may occur."
                ),
                force_flush=True,
            )
        finally:
            # Instead of time.sleep(), we're waiting on the stop event
            # in case it gets set earlier.
            self.stop_event.wait(timeout=self.error_backoff_sec)

    def _restart_stream(self):
        self._watcher = watch.Watch()
        for msg in self._watcher.stream(
            self._read_pod_log_fn,
            name=self.pod_name,
            namespace=self.namespace_name,
            timestamps=self.log_timestamps,
            follow=True,
        ):
            self._write(msg)
            # Every message check if a stop is requested.
            if self.stop_event.is_set():
                self._stop()
                return

    def _write(self, msg: str, force_flush: bool = False):
        self._out_stream.write(msg)
        self._out_stream.write("\n")
        if force_flush:
            self.flush()
        if self.log_to_stdout:
            logger.info(msg)



""" Multicast DNS Service Discovery for Python, v0.14-wmcbrine
    Copyright 2003 Paul Scott-Murphy, 2014 William McBrine

    This module provides a framework for the use of DNS Service Discovery
    using IP multicast.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
    USA
"""


import time

_float = float


def current_time_millis() -> float:
    """Current time in milliseconds.

    The current implemention uses `time.monotonic`
    but may change in the future.
    """
    return time.monotonic() * 1000


def millis_to_seconds(millis: _float) -> float:
    """Convert milliseconds to seconds."""
    return millis / 1000.0



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Thread that sends random weighted requests on a TestService stub."""

import random
import threading
import time
import traceback


def _weighted_test_case_generator(weighted_cases):
    weight_sum = sum(weighted_cases.itervalues())

    while True:
        val = random.uniform(0, weight_sum)
        partial_sum = 0
        for case in weighted_cases:
            partial_sum += weighted_cases[case]
            if val <= partial_sum:
                yield case
                break


class TestRunner(threading.Thread):
    def __init__(self, stub, test_cases, hist, exception_queue, stop_event):
        super(TestRunner, self).__init__()
        self._exception_queue = exception_queue
        self._stop_event = stop_event
        self._stub = stub
        self._test_cases = _weighted_test_case_generator(test_cases)
        self._histogram = hist

    def run(self):
        while not self._stop_event.is_set():
            try:
                test_case = next(self._test_cases)
                start_time = time.time()
                test_case.test_interoperability(self._stub, None)
                end_time = time.time()
                self._histogram.add((end_time - start_time) * 1e9)
            except Exception as e:  # pylint: disable=broad-except
                traceback.print_exc()
                self._exception_queue.put(
                    Exception(
                        "An exception occurred during test {}".format(
                            test_case
                        ),
                        e,
                    )
                )



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
from typing import List

from absl import flags
from absl.testing import absltest

from framework import xds_k8s_flags
from framework import xds_k8s_testcase
from framework import xds_url_map_testcase
from framework.helpers import skips

logger = logging.getLogger(__name__)
flags.adopt_module_key_flags(xds_k8s_testcase)
flags.mark_flag_as_required("server_image_canonical")

# Type aliases
RpcTypeUnaryCall = xds_url_map_testcase.RpcTypeUnaryCall
RpcTypeEmptyCall = xds_url_map_testcase.RpcTypeEmptyCall
_XdsTestServer = xds_k8s_testcase.XdsTestServer
_XdsTestClient = xds_k8s_testcase.XdsTestClient
_Lang = skips.Lang

# Testing consts
_QPS = 100
_REPLICA_COUNT = 5


class OutlierDetectionTest(xds_k8s_testcase.RegularXdsKubernetesTestCase):
    """
    Implementation of https://github.com/grpc/grpc/blob/master/doc/xds-test-descriptions.md#outlier_detection

    This test verifies that the client applies the outlier detection
    configuration and temporarily drops traffic to a server that fails
    requests.
    """

    @classmethod
    def setUpClass(cls):
        """Force the java test server for languages not yet supporting
        the `rpc-behavior` feature.

        https://github.com/grpc/grpc/blob/master/doc/xds-test-descriptions.md#server
        """
        super().setUpClass()
        # gRPC Java implemented server "error-code-" rpc-behavior in v1.47.x.
        if cls.lang_spec.client_lang == _Lang.JAVA:
            return

        # gRPC CPP implemented server "hostname" rpc-behavior in v1.57.x,
        # see https://github.com/grpc/grpc/pull/33446.
        if (
            cls.lang_spec.client_lang == _Lang.CPP
            and cls.lang_spec.version_gte("v1.57.x")
        ):
            return

        # gRPC go, python and node fallback to the gRPC Java.
        # TODO(https://github.com/grpc/grpc-go/issues/6288): use go server.
        # TODO(https://github.com/grpc/grpc/issues/33134): use python server.
        cls.server_image = xds_k8s_flags.SERVER_IMAGE_CANONICAL.value

    @staticmethod
    def is_supported(config: skips.TestConfig) -> bool:
        if config.client_lang in _Lang.CPP | _Lang.PYTHON:
            return config.version_gte("v1.48.x")
        if config.client_lang == _Lang.JAVA:
            return config.version_gte("v1.49.x")
        if config.client_lang == _Lang.NODE:
            return config.version_gte("v1.6.x")
        if config.client_lang == _Lang.GO:
            # TODO(zasweq): Update when the feature makes in a version branch.
            return config.version_gte("master")
        return False

    def test_outlier_detection(self) -> None:
        with self.subTest("00_create_health_check"):
            self.td.create_health_check()

        with self.subTest("01_create_backend_service"):
            self.td.create_backend_service(
                outlier_detection={
                    "interval": {"seconds": 2, "nanos": 0},
                    "successRateRequestVolume": 20,
                }
            )

        with self.subTest("02_create_url_map"):
            self.td.create_url_map(self.server_xds_host, self.server_xds_port)

        with self.subTest("03_create_target_proxy"):
            self.td.create_target_proxy()

        with self.subTest("04_create_forwarding_rule"):
            self.td.create_forwarding_rule(self.server_xds_port)

        test_servers: List[_XdsTestServer]
        with self.subTest("05_start_test_servers"):
            test_servers = self.startTestServers(replica_count=_REPLICA_COUNT)

        with self.subTest("06_add_server_backends_to_backend_services"):
            self.setupServerBackends()

        test_client: _XdsTestClient
        with self.subTest("07_start_test_client"):
            test_client = self.startTestClient(test_servers[0], qps=_QPS)

        with self.subTest("08_test_client_xds_config_exists"):
            self.assertXdsConfigExists(test_client)

        with self.subTest("09_test_servers_received_rpcs_from_test_client"):
            self.assertRpcsEventuallyGoToGivenServers(test_client, test_servers)

        rpc_types = (RpcTypeUnaryCall,)
        with self.subTest("10_chosen_server_removed_by_outlier_detection"):
            test_client.update_config.configure(
                rpc_types=rpc_types,
                metadata=(
                    (
                        RpcTypeUnaryCall,
                        "rpc-behavior",
                        f"hostname={test_servers[0].hostname} error-code-2",
                    ),
                ),
            )
            self.assertRpcsEventuallyGoToGivenServers(
                test_client, test_servers[1:]
            )

        with self.subTest("11_ejected_server_returned_after_failures_stopped"):
            test_client.update_config.configure(rpc_types=rpc_types)
            self.assertRpcsEventuallyGoToGivenServers(test_client, test_servers)


if __name__ == "__main__":
    absltest.main(failfast=True)



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
import re
import subprocess
import time
from typing import Optional

logger = logging.getLogger(__name__)


class PortForwardingError(Exception):
    """Error forwarding port"""


class PortForwarder:
    PORT_FORWARD_LOCAL_ADDRESS: str = "127.0.0.1"

    def __init__(
        self,
        context: str,
        namespace: str,
        destination: str,
        remote_port: int,
        local_port: Optional[int] = None,
        local_address: Optional[str] = None,
    ):
        self.context = context
        self.namespace = namespace
        self.destination = destination
        self.remote_port = remote_port
        self.local_address = local_address or self.PORT_FORWARD_LOCAL_ADDRESS
        self.local_port: Optional[int] = local_port
        self.subprocess: Optional[subprocess.Popen] = None

    def connect(self) -> None:
        if self.local_port:
            port_mapping = f"{self.local_port}:{self.remote_port}"
        else:
            port_mapping = f":{self.remote_port}"
        cmd = [
            "kubectl",
            "--context",
            self.context,
            "--namespace",
            self.namespace,
            "port-forward",
            "--address",
            self.local_address,
            self.destination,
            port_mapping,
        ]
        logger.debug(
            "Executing port forwarding subprocess cmd: %s", " ".join(cmd)
        )
        self.subprocess = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
        )
        # Wait for stdout line indicating successful start.
        if self.local_port:
            local_port_expected = (
                f"Forwarding from {self.local_address}:{self.local_port}"
                f" -> {self.remote_port}"
            )
        else:
            local_port_re = re.compile(
                f"Forwarding from {self.local_address}:([0-9]+) ->"
                f" {self.remote_port}"
            )
        try:
            while True:
                time.sleep(0.05)
                output = self.subprocess.stdout.readline().strip()
                if not output:
                    return_code = self.subprocess.poll()
                    if return_code is not None:
                        errors = [
                            error
                            for error in self.subprocess.stdout.readlines()
                        ]
                        raise PortForwardingError(
                            "Error forwarding port, kubectl return "
                            f"code {return_code}, output {errors}"
                        )
                    # If there is no output, and the subprocess is not exiting,
                    # continue waiting for the log line.
                    continue

                # Validate output log
                if self.local_port:
                    if output != local_port_expected:
                        raise PortForwardingError(
                            f"Error forwarding port, unexpected output {output}"
                        )
                else:
                    groups = local_port_re.search(output)
                    if groups is None:
                        raise PortForwardingError(
                            f"Error forwarding port, unexpected output {output}"
                        )
                    # Update local port to the randomly picked one
                    self.local_port = int(groups[1])

                logger.info(output)
                break
        except Exception:
            self.close()
            raise

    def close(self) -> None:
        if self.subprocess is not None:
            logger.info(
                "Shutting down port forwarding, pid %s", self.subprocess.pid
            )
            self.subprocess.kill()
            stdout, _ = self.subprocess.communicate(timeout=5)
            logger.info("Port forwarding stopped")
            logger.debug("Port forwarding remaining stdout: %s", stdout)
            self.subprocess = None



#!/usr/bin/env python3
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import collections
import operator
import os
import re
import subprocess

#
# Find the root of the git tree
#

git_root = (
    subprocess.check_output(["git", "rev-parse", "--show-toplevel"])
    .decode("utf-8")
    .strip()
)

#
# Parse command line arguments
#

default_out = os.path.join(git_root, ".github", "CODEOWNERS")

argp = argparse.ArgumentParser("Generate .github/CODEOWNERS file")
argp.add_argument(
    "--out",
    "-o",
    type=str,
    default=default_out,
    help="Output file (default %s)" % default_out,
)
args = argp.parse_args()

#
# Walk git tree to locate all OWNERS files
#

owners_files = [
    os.path.join(root, "OWNERS")
    for root, dirs, files in os.walk(git_root)
    if "OWNERS" in files
]

#
# Parse owners files
#

Owners = collections.namedtuple("Owners", "parent directives dir")
Directive = collections.namedtuple("Directive", "who globs")


def parse_owners(filename):
    with open(filename) as f:
        src = f.read().splitlines()
    parent = True
    directives = []
    for line in src:
        line = line.strip()
        # line := directive | comment
        if not line:
            continue
        if line[0] == "#":
            continue
        # it's a directive
        directive = None
        if line == "set noparent":
            parent = False
        elif line == "*":
            directive = Directive(who="*", globs=[])
        elif " " in line:
            (who, globs) = line.split(" ", 1)
            globs_list = [glob for glob in globs.split(" ") if glob]
            directive = Directive(who=who, globs=globs_list)
        else:
            directive = Directive(who=line, globs=[])
        if directive:
            directives.append(directive)
    return Owners(
        parent=parent,
        directives=directives,
        dir=os.path.relpath(os.path.dirname(filename), git_root),
    )


owners_data = sorted(
    [parse_owners(filename) for filename in owners_files],
    key=operator.attrgetter("dir"),
)

#
# Modify owners so that parented OWNERS files point to the actual
# Owners tuple with their parent field
#

new_owners_data = []
for owners in owners_data:
    if owners.parent == True:
        best_parent = None
        best_parent_score = None
        for possible_parent in owners_data:
            if possible_parent is owners:
                continue
            rel = os.path.relpath(owners.dir, possible_parent.dir)
            # '..' ==> we had to walk up from possible_parent to get to owners
            #      ==> not a parent
            if ".." in rel:
                continue
            depth = len(rel.split(os.sep))
            if not best_parent or depth < best_parent_score:
                best_parent = possible_parent
                best_parent_score = depth
        if best_parent:
            owners = owners._replace(parent=best_parent.dir)
        else:
            owners = owners._replace(parent=None)
    new_owners_data.append(owners)
owners_data = new_owners_data

#
# In bottom to top order, process owners data structures to build up
# a CODEOWNERS file for GitHub
#


def full_dir(rules_dir, sub_path):
    return os.path.join(rules_dir, sub_path) if rules_dir != "." else sub_path


# glob using git
gg_cache = {}


def git_glob(glob):
    global gg_cache
    if glob in gg_cache:
        return gg_cache[glob]
    r = set(
        subprocess.check_output(
            ["git", "ls-files", os.path.join(git_root, glob)]
        )
        .decode("utf-8")
        .strip()
        .splitlines()
    )
    gg_cache[glob] = r
    return r


def expand_directives(root, directives):
    globs = collections.OrderedDict()
    # build a table of glob --> owners
    for directive in directives:
        for glob in directive.globs or ["**"]:
            if glob not in globs:
                globs[glob] = []
            if directive.who not in globs[glob]:
                globs[glob].append(directive.who)
    # expand owners for intersecting globs
    sorted_globs = sorted(
        list(globs.keys()),
        key=lambda g: len(git_glob(full_dir(root, g))),
        reverse=True,
    )
    out_globs = collections.OrderedDict()
    for glob_add in sorted_globs:
        who_add = globs[glob_add]
        pre_items = [i for i in list(out_globs.items())]
        out_globs[glob_add] = who_add.copy()
        for glob_have, who_have in pre_items:
            files_add = git_glob(full_dir(root, glob_add))
            files_have = git_glob(full_dir(root, glob_have))
            intersect = files_have.intersection(files_add)
            if intersect:
                for f in sorted(files_add):  # sorted to ensure merge stability
                    if f not in intersect:
                        out_globs[os.path.relpath(f, start=root)] = who_add
                for who in who_have:
                    if who not in out_globs[glob_add]:
                        out_globs[glob_add].append(who)
    return out_globs


def add_parent_to_globs(parent, globs, globs_dir):
    if not parent:
        return
    for owners in owners_data:
        if owners.dir == parent:
            owners_globs = expand_directives(owners.dir, owners.directives)
            for oglob, oglob_who in list(owners_globs.items()):
                for gglob, gglob_who in list(globs.items()):
                    files_parent = git_glob(full_dir(owners.dir, oglob))
                    files_child = git_glob(full_dir(globs_dir, gglob))
                    intersect = files_parent.intersection(files_child)
                    gglob_who_orig = gglob_who.copy()
                    if intersect:
                        for f in sorted(
                            files_child
                        ):  # sorted to ensure merge stability
                            if f not in intersect:
                                who = gglob_who_orig.copy()
                                globs[os.path.relpath(f, start=globs_dir)] = who
                        for who in oglob_who:
                            if who not in gglob_who:
                                gglob_who.append(who)
            add_parent_to_globs(owners.parent, globs, globs_dir)
            return
    assert False


todo = owners_data.copy()
done = set()
with open(args.out, "w") as out:
    out.write("# Auto-generated by the tools/mkowners/mkowners.py tool\n")
    out.write("# Uses OWNERS files in different modules throughout the\n")
    out.write("# repository as the source of truth for module ownership.\n")
    written_globs = []
    while todo:
        head, *todo = todo
        if head.parent and not head.parent in done:
            todo.append(head)
            continue
        globs = expand_directives(head.dir, head.directives)
        add_parent_to_globs(head.parent, globs, head.dir)
        for glob, owners in list(globs.items()):
            skip = False
            for glob1, owners1, dir1 in reversed(written_globs):
                files = git_glob(full_dir(head.dir, glob))
                files1 = git_glob(full_dir(dir1, glob1))
                intersect = files.intersection(files1)
                if files == intersect:
                    if sorted(owners) == sorted(owners1):
                        skip = True  # nothing new in this rule
                        break
                elif intersect:
                    # continuing would cause a semantic change since some files are
                    # affected differently by this rule and CODEOWNERS is order dependent
                    break
            if not skip:
                out.write(
                    "/%s %s\n" % (full_dir(head.dir, glob), " ".join(owners))
                )
                written_globs.append((glob, owners, head.dir))
        done.add(head.dir)



# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The module contains helpers to enable color output in terminals.

Use this to log resources dumped as a structured document (f.e. YAML),
and enable colorful syntax highlighting.

TODO(sergiitk): This can be used to output protobuf responses formatted as JSON.
"""
import logging
from typing import Optional

from absl import flags
import pygments
import pygments.formatter
import pygments.formatters.other
import pygments.formatters.terminal
import pygments.formatters.terminal256
import pygments.lexer
import pygments.lexers.data
import pygments.styles

# The style for terminals supporting 8/16 colors.
STYLE_ANSI_16 = "ansi16"
# Join with pygments styles for terminals supporting 88/256 colors.
ALL_COLOR_STYLES = [STYLE_ANSI_16] + list(pygments.styles.get_all_styles())

# Flags.
COLOR = flags.DEFINE_bool("color", default=True, help="Colorize the output")
COLOR_STYLE = flags.DEFINE_enum(
    "color_style",
    default="material",
    enum_values=ALL_COLOR_STYLES,
    help=(
        "Color styles for terminals supporting 256 colors. "
        f"Use {STYLE_ANSI_16} style for terminals supporting 8/16 colors"
    ),
)

logger = logging.getLogger(__name__)

# Type aliases.
Lexer = pygments.lexer.Lexer
YamlLexer = pygments.lexers.data.YamlLexer
Formatter = pygments.formatter.Formatter
NullFormatter = pygments.formatters.other.NullFormatter
TerminalFormatter = pygments.formatters.terminal.TerminalFormatter
Terminal256Formatter = pygments.formatters.terminal256.Terminal256Formatter


class Highlighter:
    formatter: Formatter
    lexer: Lexer
    color: bool
    color_style: Optional[str] = None

    def __init__(
        self,
        *,
        lexer: Lexer,
        color: Optional[bool] = None,
        color_style: Optional[str] = None,
    ):
        self.lexer = lexer
        self.color = color if color is not None else COLOR.value

        if self.color:
            color_style = color_style if color_style else COLOR_STYLE.value
            if color_style not in ALL_COLOR_STYLES:
                raise ValueError(
                    f"Unrecognized color style {color_style}, "
                    f"valid styles: {ALL_COLOR_STYLES}"
                )
            if color_style == STYLE_ANSI_16:
                # 8/16 colors support only.
                self.formatter = TerminalFormatter()
            else:
                # 88/256 colors.
                self.formatter = Terminal256Formatter(style=color_style)
        else:
            self.formatter = NullFormatter()

    def highlight(self, code: str) -> str:
        return pygments.highlight(code, self.lexer, self.formatter)


class HighlighterYaml(Highlighter):
    def __init__(
        self, *, color: Optional[bool] = None, color_style: Optional[str] = None
    ):
        super().__init__(
            lexer=YamlLexer(encoding="utf-8"),
            color=color,
            color_style=color_style,
        )



#!/usr/bin/env python3
# Copyright 2023 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.
"""Certificate chain of length 3 with requireExplicitPolicy=3 on the root,
meaning an explicit policy should not be required and the chain should verify
successfully regardless of if the root constraints are enforced."""

import sys
sys.path += ['../..']

import gencerts

# Self-signed root certificate.
root = gencerts.create_self_signed_root_certificate('Root')
root.get_extensions().set_property('policyConstraints',
                                   'critical,requireExplicitPolicy:3')

# Intermediate certificate.
intermediate = gencerts.create_intermediate_certificate('Intermediate', root)

# Target certificate.
target = gencerts.create_end_entity_certificate('Target', intermediate)

chain = [target, intermediate, root]
gencerts.write_chain(__doc__, chain, 'chain.pem')



# Copyright 2018 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import datetime
import json
import os
import sys
import time
import traceback

import jwt
import requests

_GITHUB_API_PREFIX = "https://api.github.com"
_GITHUB_REPO = "grpc/grpc"
_GITHUB_APP_ID = 22338
_INSTALLATION_ID = 519109

_ACCESS_TOKEN_CACHE = None
_ACCESS_TOKEN_FETCH_RETRIES = 6
_ACCESS_TOKEN_FETCH_RETRIES_INTERVAL_S = 15

_CHANGE_LABELS = {
    -1: "improvement",
    0: "none",
    1: "low",
    2: "medium",
    3: "high",
}

_INCREASE_DECREASE = {
    -1: "decrease",
    0: "neutral",
    1: "increase",
}


def _jwt_token():
    github_app_key = open(
        os.path.join(
            os.environ["KOKORO_KEYSTORE_DIR"], "73836_grpc_checks_private_key"
        ),
        "rb",
    ).read()
    return jwt.encode(
        {
            "iat": int(time.time()),
            "exp": int(time.time() + 60 * 10),  # expire in 10 minutes
            "iss": _GITHUB_APP_ID,
        },
        github_app_key,
        algorithm="RS256",
    )


def _access_token():
    global _ACCESS_TOKEN_CACHE
    if _ACCESS_TOKEN_CACHE == None or _ACCESS_TOKEN_CACHE["exp"] < time.time():
        for i in range(_ACCESS_TOKEN_FETCH_RETRIES):
            resp = requests.post(
                url="https://api.github.com/app/installations/%s/access_tokens"
                % _INSTALLATION_ID,
                headers={
                    "Authorization": "Bearer %s" % _jwt_token(),
                    "Accept": "application/vnd.github.machine-man-preview+json",
                },
            )

            try:
                _ACCESS_TOKEN_CACHE = {
                    "token": resp.json()["token"],
                    "exp": time.time() + 60,
                }
                break
            except (KeyError, ValueError):
                traceback.print_exc()
                print("HTTP Status %d %s" % (resp.status_code, resp.reason))
                print("Fetch access token from Github API failed:")
                print(resp.text)
                if i != _ACCESS_TOKEN_FETCH_RETRIES - 1:
                    print(
                        "Retrying after %.2f second."
                        % _ACCESS_TOKEN_FETCH_RETRIES_INTERVAL_S
                    )
                    time.sleep(_ACCESS_TOKEN_FETCH_RETRIES_INTERVAL_S)
        else:
            print("error: Unable to fetch access token, exiting...")
            sys.exit(0)

    return _ACCESS_TOKEN_CACHE["token"]


def _call(url, method="GET", json=None):
    if not url.startswith("https://"):
        url = _GITHUB_API_PREFIX + url
    headers = {
        "Authorization": "Bearer %s" % _access_token(),
        "Accept": "application/vnd.github.antiope-preview+json",
    }
    return requests.request(method=method, url=url, headers=headers, json=json)


def _latest_commit():
    resp = _call(
        "/repos/%s/pulls/%s/commits"
        % (_GITHUB_REPO, os.environ["KOKORO_GITHUB_PULL_REQUEST_NUMBER"])
    )
    return resp.json()[-1]


def check_on_pr(name, summary, success=True):
    """Create/Update a check on current pull request.

    The check runs are aggregated by their name, so newer check will update the
    older check with the same name.

    Requires environment variable 'KOKORO_GITHUB_PULL_REQUEST_NUMBER' to indicate which pull request
    should be updated.

    Args:
      name: The name of the check.
      summary: A str in Markdown to be used as the detail information of the check.
      success: A bool indicates whether the check is succeed or not.
    """
    if "KOKORO_GIT_COMMIT" not in os.environ:
        print("Missing KOKORO_GIT_COMMIT env var: not checking")
        return
    if "KOKORO_KEYSTORE_DIR" not in os.environ:
        print("Missing KOKORO_KEYSTORE_DIR env var: not checking")
        return
    if "KOKORO_GITHUB_PULL_REQUEST_NUMBER" not in os.environ:
        print("Missing KOKORO_GITHUB_PULL_REQUEST_NUMBER env var: not checking")
        return
    MAX_SUMMARY_LEN = 65400
    if len(summary) > MAX_SUMMARY_LEN:
        # Drop some hints to the log should someone come looking for what really happened!
        print("Clipping too long summary")
        print(summary)
        summary = summary[:MAX_SUMMARY_LEN] + "\n\n\n... CLIPPED (too long)"
    completion_time = (
        str(datetime.datetime.utcnow().replace(microsecond=0).isoformat()) + "Z"
    )
    resp = _call(
        "/repos/%s/check-runs" % _GITHUB_REPO,
        method="POST",
        json={
            "name": name,
            "head_sha": os.environ["KOKORO_GIT_COMMIT"],
            "status": "completed",
            "completed_at": completion_time,
            "conclusion": "success" if success else "failure",
            "output": {
                "title": name,
                "summary": summary,
            },
        },
    )
    print(
        "Result of Creating/Updating Check on PR:",
        json.dumps(resp.json(), indent=2),
    )


def label_significance_on_pr(name, change, labels=_CHANGE_LABELS):
    """Add a label to the PR indicating the significance of the check.

    Requires environment variable 'KOKORO_GITHUB_PULL_REQUEST_NUMBER' to indicate which pull request
    should be updated.

    Args:
      name: The name of the label.
      value: A str in Markdown to be used as the detail information of the label.
    """
    if change < min(list(labels.keys())):
        change = min(list(labels.keys()))
    if change > max(list(labels.keys())):
        change = max(list(labels.keys()))
    value = labels[change]
    if "KOKORO_GIT_COMMIT" not in os.environ:
        print("Missing KOKORO_GIT_COMMIT env var: not checking")
        return
    if "KOKORO_KEYSTORE_DIR" not in os.environ:
        print("Missing KOKORO_KEYSTORE_DIR env var: not checking")
        return
    if "KOKORO_GITHUB_PULL_REQUEST_NUMBER" not in os.environ:
        print("Missing KOKORO_GITHUB_PULL_REQUEST_NUMBER env var: not checking")
        return
    existing = _call(
        "/repos/%s/issues/%s/labels"
        % (_GITHUB_REPO, os.environ["KOKORO_GITHUB_PULL_REQUEST_NUMBER"]),
        method="GET",
    ).json()
    print("Result of fetching labels on PR:", existing)
    new = [x["name"] for x in existing if not x["name"].startswith(name + "/")]
    new.append(name + "/" + value)
    resp = _call(
        "/repos/%s/issues/%s/labels"
        % (_GITHUB_REPO, os.environ["KOKORO_GITHUB_PULL_REQUEST_NUMBER"]),
        method="PUT",
        json=new,
    )
    print("Result of setting labels on PR:", resp.text)


def label_increase_decrease_on_pr(name, change, significant):
    if change <= -significant:
        label_significance_on_pr(name, -1, _INCREASE_DECREASE)
    elif change >= significant:
        label_significance_on_pr(name, 1, _INCREASE_DECREASE)
    else:
        label_significance_on_pr(name, 0, _INCREASE_DECREASE)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from concurrent import futures
import time
import unittest

import grpc
from grpc.framework.foundation import logging_pool
import grpc_testing

from tests.testing import _application_common
from tests.testing import _application_testing_common
from tests.testing import _client_application
from tests.testing.proto import requests_pb2
from tests.testing.proto import services_pb2
from tests.unit.framework.common import test_constants


# TODO(https://github.com/protocolbuffers/protobuf/issues/3452): Drop this skip.
@unittest.skipIf(
    services_pb2.DESCRIPTOR.services_by_name.get("FirstService") is None,
    "Fix protobuf issue 3452!",
)
class ClientTest(unittest.TestCase):
    def setUp(self):
        # In this test the client-side application under test executes in
        # a separate thread while we retain use of the test thread to "play
        # server".
        self._client_execution_thread_pool = logging_pool.pool(1)

        self._fake_time = grpc_testing.strict_fake_time(time.time())
        self._real_time = grpc_testing.strict_real_time()
        self._fake_time_channel = grpc_testing.channel(
            services_pb2.DESCRIPTOR.services_by_name.values(), self._fake_time
        )
        self._real_time_channel = grpc_testing.channel(
            services_pb2.DESCRIPTOR.services_by_name.values(), self._real_time
        )

    def tearDown(self):
        self._client_execution_thread_pool.shutdown(wait=True)

    def test_successful_unary_unary(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.UNARY_UNARY,
            self._real_time_channel,
        )
        (
            invocation_metadata,
            request,
            rpc,
        ) = self._real_time_channel.take_unary_unary(
            _application_testing_common.FIRST_SERVICE_UNUN
        )
        rpc.send_initial_metadata(())
        rpc.terminate(
            _application_common.UNARY_UNARY_RESPONSE, (), grpc.StatusCode.OK, ""
        )
        application_return_value = application_future.result()

        self.assertEqual(_application_common.UNARY_UNARY_REQUEST, request)
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.SATISFACTORY,
        )

    def test_successful_unary_stream(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.UNARY_STREAM,
            self._fake_time_channel,
        )
        (
            invocation_metadata,
            request,
            rpc,
        ) = self._fake_time_channel.take_unary_stream(
            _application_testing_common.FIRST_SERVICE_UNSTRE
        )
        rpc.send_initial_metadata(())
        rpc.terminate((), grpc.StatusCode.OK, "")
        application_return_value = application_future.result()

        self.assertEqual(_application_common.UNARY_STREAM_REQUEST, request)
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.SATISFACTORY,
        )

    def test_successful_stream_unary(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.STREAM_UNARY,
            self._real_time_channel,
        )
        invocation_metadata, rpc = self._real_time_channel.take_stream_unary(
            _application_testing_common.FIRST_SERVICE_STREUN
        )
        rpc.send_initial_metadata(())
        first_request = rpc.take_request()
        second_request = rpc.take_request()
        third_request = rpc.take_request()
        rpc.requests_closed()
        rpc.terminate(
            _application_common.STREAM_UNARY_RESPONSE,
            (),
            grpc.StatusCode.OK,
            "",
        )
        application_return_value = application_future.result()

        self.assertEqual(
            _application_common.STREAM_UNARY_REQUEST, first_request
        )
        self.assertEqual(
            _application_common.STREAM_UNARY_REQUEST, second_request
        )
        self.assertEqual(
            _application_common.STREAM_UNARY_REQUEST, third_request
        )
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.SATISFACTORY,
        )

    def test_successful_stream_stream(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.STREAM_STREAM,
            self._fake_time_channel,
        )
        invocation_metadata, rpc = self._fake_time_channel.take_stream_stream(
            _application_testing_common.FIRST_SERVICE_STRESTRE
        )
        first_request = rpc.take_request()
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        second_request = rpc.take_request()
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.requests_closed()
        rpc.terminate((), grpc.StatusCode.OK, "")
        application_return_value = application_future.result()

        self.assertEqual(
            _application_common.STREAM_STREAM_REQUEST, first_request
        )
        self.assertEqual(
            _application_common.STREAM_STREAM_REQUEST, second_request
        )
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.SATISFACTORY,
        )

    def test_concurrent_stream_stream(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.CONCURRENT_STREAM_STREAM,
            self._real_time_channel,
        )
        rpcs = []
        for _ in range(test_constants.RPC_CONCURRENCY):
            (
                invocation_metadata,
                rpc,
            ) = self._real_time_channel.take_stream_stream(
                _application_testing_common.FIRST_SERVICE_STRESTRE
            )
            rpcs.append(rpc)
        requests = {}
        for rpc in rpcs:
            requests[rpc] = [rpc.take_request()]
        for rpc in rpcs:
            rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
            rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        for rpc in rpcs:
            requests[rpc].append(rpc.take_request())
        for rpc in rpcs:
            rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
            rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        for rpc in rpcs:
            rpc.requests_closed()
        for rpc in rpcs:
            rpc.terminate((), grpc.StatusCode.OK, "")
        application_return_value = application_future.result()

        for requests_of_one_rpc in requests.values():
            for request in requests_of_one_rpc:
                self.assertEqual(
                    _application_common.STREAM_STREAM_REQUEST, request
                )
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.SATISFACTORY,
        )

    def test_cancelled_unary_unary(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.CANCEL_UNARY_UNARY,
            self._fake_time_channel,
        )
        (
            invocation_metadata,
            request,
            rpc,
        ) = self._fake_time_channel.take_unary_unary(
            _application_testing_common.FIRST_SERVICE_UNUN
        )
        rpc.send_initial_metadata(())
        rpc.cancelled()
        application_return_value = application_future.result()

        self.assertEqual(_application_common.UNARY_UNARY_REQUEST, request)
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.SATISFACTORY,
        )

    def test_status_stream_unary(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.CONCURRENT_STREAM_UNARY,
            self._fake_time_channel,
        )
        rpcs = tuple(
            self._fake_time_channel.take_stream_unary(
                _application_testing_common.FIRST_SERVICE_STREUN
            )[1]
            for _ in range(test_constants.THREAD_CONCURRENCY)
        )
        for rpc in rpcs:
            rpc.take_request()
            rpc.take_request()
            rpc.take_request()
            rpc.requests_closed()
            rpc.send_initial_metadata(
                (
                    (
                        "my_metadata_key",
                        "My Metadata Value!",
                    ),
                )
            )
        for rpc in rpcs[:-1]:
            rpc.terminate(
                _application_common.STREAM_UNARY_RESPONSE,
                (),
                grpc.StatusCode.OK,
                "",
            )
        rpcs[-1].terminate(
            _application_common.STREAM_UNARY_RESPONSE,
            (),
            grpc.StatusCode.RESOURCE_EXHAUSTED,
            "nope; not able to handle all those RPCs!",
        )
        application_return_value = application_future.result()

        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.UNSATISFACTORY,
        )

    def test_status_stream_stream(self):
        code = grpc.StatusCode.DEADLINE_EXCEEDED
        details = "test deadline exceeded!"

        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.STREAM_STREAM,
            self._real_time_channel,
        )
        invocation_metadata, rpc = self._real_time_channel.take_stream_stream(
            _application_testing_common.FIRST_SERVICE_STRESTRE
        )
        first_request = rpc.take_request()
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        second_request = rpc.take_request()
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.requests_closed()
        rpc.terminate((), code, details)
        application_return_value = application_future.result()

        self.assertEqual(
            _application_common.STREAM_STREAM_REQUEST, first_request
        )
        self.assertEqual(
            _application_common.STREAM_STREAM_REQUEST, second_request
        )
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.RPC_ERROR,
        )
        self.assertIs(application_return_value.code, code)
        self.assertEqual(application_return_value.details, details)

    def test_misbehaving_server_unary_unary(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.UNARY_UNARY,
            self._fake_time_channel,
        )
        (
            invocation_metadata,
            request,
            rpc,
        ) = self._fake_time_channel.take_unary_unary(
            _application_testing_common.FIRST_SERVICE_UNUN
        )
        rpc.send_initial_metadata(())
        rpc.terminate(
            _application_common.ERRONEOUS_UNARY_UNARY_RESPONSE,
            (),
            grpc.StatusCode.OK,
            "",
        )
        application_return_value = application_future.result()

        self.assertEqual(_application_common.UNARY_UNARY_REQUEST, request)
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.UNSATISFACTORY,
        )

    def test_misbehaving_server_stream_stream(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.STREAM_STREAM,
            self._real_time_channel,
        )
        invocation_metadata, rpc = self._real_time_channel.take_stream_stream(
            _application_testing_common.FIRST_SERVICE_STRESTRE
        )
        first_request = rpc.take_request()
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        second_request = rpc.take_request()
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.send_response(_application_common.STREAM_STREAM_RESPONSE)
        rpc.requests_closed()
        rpc.terminate((), grpc.StatusCode.OK, "")
        application_return_value = application_future.result()

        self.assertEqual(
            _application_common.STREAM_STREAM_REQUEST, first_request
        )
        self.assertEqual(
            _application_common.STREAM_STREAM_REQUEST, second_request
        )
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.UNSATISFACTORY,
        )

    def test_infinite_request_stream_real_time(self):
        application_future = self._client_execution_thread_pool.submit(
            _client_application.run,
            _client_application.Scenario.INFINITE_REQUEST_STREAM,
            self._real_time_channel,
        )
        invocation_metadata, rpc = self._real_time_channel.take_stream_unary(
            _application_testing_common.FIRST_SERVICE_STREUN
        )
        rpc.send_initial_metadata(())
        first_request = rpc.take_request()
        second_request = rpc.take_request()
        third_request = rpc.take_request()
        self._real_time.sleep_for(
            _application_common.INFINITE_REQUEST_STREAM_TIMEOUT
        )
        rpc.terminate(
            _application_common.STREAM_UNARY_RESPONSE,
            (),
            grpc.StatusCode.DEADLINE_EXCEEDED,
            "",
        )
        application_return_value = application_future.result()

        self.assertEqual(
            _application_common.STREAM_UNARY_REQUEST, first_request
        )
        self.assertEqual(
            _application_common.STREAM_UNARY_REQUEST, second_request
        )
        self.assertEqual(
            _application_common.STREAM_UNARY_REQUEST, third_request
        )
        self.assertIs(
            application_return_value.kind,
            _client_application.Outcome.Kind.SATISFACTORY,
        )


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python3
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Run test matrix."""

from __future__ import print_function

import argparse
import multiprocessing
import os
import sys

from python_utils.filter_pull_request_tests import filter_tests
import python_utils.jobset as jobset
import python_utils.report_utils as report_utils

_ROOT = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), "../.."))
os.chdir(_ROOT)

_DEFAULT_RUNTESTS_TIMEOUT = 1 * 60 * 60

# C/C++ tests can take long time
_CPP_RUNTESTS_TIMEOUT = 4 * 60 * 60

# Set timeout high for ObjC for Cocoapods to install pods
_OBJC_RUNTESTS_TIMEOUT = 2 * 60 * 60

# Number of jobs assigned to each run_tests.py instance
_DEFAULT_INNER_JOBS = 2

# Name of the top-level umbrella report that includes all the run_tests.py invocations
# Note that the starting letter 't' matters so that the targets are listed AFTER
# the per-test breakdown items that start with 'run_tests/' (it is more readable that way)
_MATRIX_REPORT_NAME = "toplevel_run_tests_invocations"


def _safe_report_name(name):
    """Reports with '+' in target name won't show correctly in ResultStore"""
    return name.replace("+", "p")


def _report_filename(name):
    """Generates report file name with directory structure that leads to better presentation by internal CI"""
    # 'sponge_log.xml' suffix must be there for results to get recognized by kokoro.
    return "%s/%s" % (_safe_report_name(name), "sponge_log.xml")


def _matrix_job_logfilename(shortname_for_multi_target):
    """Generate location for log file that will match the sponge_log.xml from the top-level matrix report."""
    # 'sponge_log.log' suffix must be there for log to get recognized as "target log"
    # for the corresponding 'sponge_log.xml' report.
    # the shortname_for_multi_target component must be set to match the sponge_log.xml location
    # because the top-level render_junit_xml_report is called with multi_target=True
    sponge_log_name = "%s/%s/%s" % (
        _MATRIX_REPORT_NAME,
        shortname_for_multi_target,
        "sponge_log.log",
    )
    # env variable can be used to override the base location for the reports
    # so we need to match that behavior here too
    base_dir = os.getenv("GRPC_TEST_REPORT_BASE_DIR", None)
    if base_dir:
        sponge_log_name = os.path.join(base_dir, sponge_log_name)
    return sponge_log_name


def _docker_jobspec(
    name,
    runtests_args=[],
    runtests_envs={},
    inner_jobs=_DEFAULT_INNER_JOBS,
    timeout_seconds=None,
):
    """Run a single instance of run_tests.py in a docker container"""
    if not timeout_seconds:
        timeout_seconds = _DEFAULT_RUNTESTS_TIMEOUT
    shortname = "run_tests_%s" % name
    test_job = jobset.JobSpec(
        cmdline=[
            "python3",
            "tools/run_tests/run_tests.py",
            "--use_docker",
            "-t",
            "-j",
            str(inner_jobs),
            "-x",
            "run_tests/%s" % _report_filename(name),
            "--report_suite_name",
            "%s" % _safe_report_name(name),
        ]
        + runtests_args,
        environ=runtests_envs,
        shortname=shortname,
        timeout_seconds=timeout_seconds,
        logfilename=_matrix_job_logfilename(shortname),
    )
    return test_job


def _workspace_jobspec(
    name,
    runtests_args=[],
    workspace_name=None,
    runtests_envs={},
    inner_jobs=_DEFAULT_INNER_JOBS,
    timeout_seconds=None,
):
    """Run a single instance of run_tests.py in a separate workspace"""
    if not workspace_name:
        workspace_name = "workspace_%s" % name
    if not timeout_seconds:
        timeout_seconds = _DEFAULT_RUNTESTS_TIMEOUT
    shortname = "run_tests_%s" % name
    env = {"WORKSPACE_NAME": workspace_name}
    env.update(runtests_envs)
    # if report base dir is set, we don't need to ".." to come out of the workspace dir
    report_dir_prefix = (
        "" if os.getenv("GRPC_TEST_REPORT_BASE_DIR", None) else "../"
    )
    test_job = jobset.JobSpec(
        cmdline=[
            "bash",
            "tools/run_tests/helper_scripts/run_tests_in_workspace.sh",
            "-t",
            "-j",
            str(inner_jobs),
            "-x",
            "%srun_tests/%s" % (report_dir_prefix, _report_filename(name)),
            "--report_suite_name",
            "%s" % _safe_report_name(name),
        ]
        + runtests_args,
        environ=env,
        shortname=shortname,
        timeout_seconds=timeout_seconds,
        logfilename=_matrix_job_logfilename(shortname),
    )
    return test_job


def _generate_jobs(
    languages,
    configs,
    platforms,
    iomgr_platforms=["native"],
    arch=None,
    compiler=None,
    labels=[],
    extra_args=[],
    extra_envs={},
    inner_jobs=_DEFAULT_INNER_JOBS,
    timeout_seconds=None,
):
    result = []
    for language in languages:
        for platform in platforms:
            for iomgr_platform in iomgr_platforms:
                for config in configs:
                    name = "%s_%s_%s_%s" % (
                        language,
                        platform,
                        config,
                        iomgr_platform,
                    )
                    runtests_args = [
                        "-l",
                        language,
                        "-c",
                        config,
                        "--iomgr_platform",
                        iomgr_platform,
                    ]
                    if arch or compiler:
                        name += "_%s_%s" % (arch, compiler)
                        runtests_args += [
                            "--arch",
                            arch,
                            "--compiler",
                            compiler,
                        ]
                    if "--build_only" in extra_args:
                        name += "_buildonly"
                    for extra_env in extra_envs:
                        name += "_%s_%s" % (extra_env, extra_envs[extra_env])

                    runtests_args += extra_args
                    if platform == "linux":
                        job = _docker_jobspec(
                            name=name,
                            runtests_args=runtests_args,
                            runtests_envs=extra_envs,
                            inner_jobs=inner_jobs,
                            timeout_seconds=timeout_seconds,
                        )
                    else:
                        job = _workspace_jobspec(
                            name=name,
                            runtests_args=runtests_args,
                            runtests_envs=extra_envs,
                            inner_jobs=inner_jobs,
                            timeout_seconds=timeout_seconds,
                        )

                    job.labels = [
                        platform,
                        config,
                        language,
                        iomgr_platform,
                    ] + labels
                    result.append(job)
    return result


def _create_test_jobs(extra_args=[], inner_jobs=_DEFAULT_INNER_JOBS):
    test_jobs = []
    # sanity tests
    test_jobs += _generate_jobs(
        languages=["sanity", "clang-tidy", "iwyu"],
        configs=["dbg"],
        platforms=["linux"],
        labels=["basictests"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
    )

    # supported on all platforms.
    test_jobs += _generate_jobs(
        languages=["c"],
        configs=["dbg", "opt"],
        platforms=["linux", "macos", "windows"],
        labels=["basictests", "corelang"],
        extra_args=extra_args,  # don't use multi_target report because C has too many test cases
        inner_jobs=inner_jobs,
        timeout_seconds=_CPP_RUNTESTS_TIMEOUT,
    )

    # C# tests (both on .NET desktop/mono and .NET core)
    test_jobs += _generate_jobs(
        languages=["csharp"],
        configs=["dbg", "opt"],
        platforms=["linux", "macos", "windows"],
        labels=["basictests", "multilang"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
    )

    # ARM64 Linux C# tests
    test_jobs += _generate_jobs(
        languages=["csharp"],
        configs=["dbg", "opt"],
        platforms=["linux"],
        arch="arm64",
        compiler="default",
        labels=["basictests_arm64"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
    )

    test_jobs += _generate_jobs(
        languages=["python"],
        configs=["opt"],
        platforms=["linux", "macos", "windows"],
        iomgr_platforms=["native"],
        labels=["basictests", "multilang"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
    )

    # ARM64 Linux Python tests
    test_jobs += _generate_jobs(
        languages=["python"],
        configs=["opt"],
        platforms=["linux"],
        arch="arm64",
        compiler="default",
        iomgr_platforms=["native"],
        labels=["basictests_arm64"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
    )

    # supported on linux and mac.
    test_jobs += _generate_jobs(
        languages=["c++"],
        configs=["dbg", "opt"],
        platforms=["linux", "macos"],
        labels=["basictests", "corelang"],
        extra_args=extra_args,  # don't use multi_target report because C++ has too many test cases
        inner_jobs=inner_jobs,
        timeout_seconds=_CPP_RUNTESTS_TIMEOUT,
    )

    test_jobs += _generate_jobs(
        languages=["ruby", "php7"],
        configs=["dbg", "opt"],
        platforms=["linux", "macos"],
        labels=["basictests", "multilang"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
    )

    # ARM64 Linux Ruby and PHP tests
    test_jobs += _generate_jobs(
        languages=["ruby", "php7"],
        configs=["dbg", "opt"],
        platforms=["linux"],
        arch="arm64",
        compiler="default",
        labels=["basictests_arm64"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
    )

    # supported on mac only.
    test_jobs += _generate_jobs(
        languages=["objc"],
        configs=["opt"],
        platforms=["macos"],
        labels=["basictests", "multilang"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
        timeout_seconds=_OBJC_RUNTESTS_TIMEOUT,
    )

    return test_jobs


def _create_portability_test_jobs(
    extra_args=[], inner_jobs=_DEFAULT_INNER_JOBS
):
    test_jobs = []
    # portability C x86
    test_jobs += _generate_jobs(
        languages=["c"],
        configs=["dbg"],
        platforms=["linux"],
        arch="x86",
        compiler="default",
        labels=["portability", "corelang"],
        extra_args=extra_args,
        inner_jobs=inner_jobs,
    )

    # portability C and C++ on x64
    for compiler in [
        "gcc8",
        # 'gcc10.2_openssl102', // TODO(b/283304471): Enable this later
        "gcc12",
        "gcc12_openssl309",
        "gcc_musl",
        "clang6",
        "clang15",
    ]:
        test_jobs += _generate_jobs(
            languages=["c", "c++"],
            configs=["dbg"],
            platforms=["linux"],
            arch="x64",
            compiler=compiler,
            labels=["portability", "corelang"],
            extra_args=extra_args,
            inner_jobs=inner_jobs,
            timeout_seconds=_CPP_RUNTESTS_TIMEOUT,
        )

    # portability C on Windows 64-bit (x86 is the default)
    test_jobs += _generate_jobs(
        languages=["c"],
        configs=["dbg"],
        platforms=["windows"],
        arch="x64",
        compiler="default",
        labels=["portability", "corelang"],
        extra_args=extra_args,
        inner_jobs=inner_jobs,
    )

    # portability C on Windows with the "Visual Studio" cmake
    # generator, i.e. not using Ninja (to verify that we can still build with msbuild)
    test_jobs += _generate_jobs(
        languages=["c"],
        configs=["dbg"],
        platforms=["windows"],
        arch="default",
        compiler="cmake_vs2019",
        labels=["portability", "corelang"],
        extra_args=extra_args,
        inner_jobs=inner_jobs,
    )

    # portability C++ on Windows
    # TODO(jtattermusch): some of the tests are failing, so we force --build_only
    test_jobs += _generate_jobs(
        languages=["c++"],
        configs=["dbg"],
        platforms=["windows"],
        arch="default",
        compiler="default",
        labels=["portability", "corelang"],
        extra_args=extra_args + ["--build_only"],
        inner_jobs=inner_jobs,
        timeout_seconds=_CPP_RUNTESTS_TIMEOUT,
    )

    # portability C and C++ on Windows using VS2019 (build only)
    # TODO(jtattermusch): The C tests with exactly the same config are already running as part of the
    # basictests_c suite (so we force --build_only to avoid running them twice).
    # The C++ tests aren't all passing, so also force --build_only.
    # NOTE(veblush): This is not neded as default=cmake_ninja_vs2019
    # test_jobs += _generate_jobs(
    #     languages=["c", "c++"],
    #     configs=["dbg"],
    #     platforms=["windows"],
    #     arch="x64",
    #     compiler="cmake_ninja_vs2019",
    #     labels=["portability", "corelang"],
    #     extra_args=extra_args + ["--build_only"],
    #     inner_jobs=inner_jobs,
    #     timeout_seconds=_CPP_RUNTESTS_TIMEOUT,
    # )

    # C and C++ with no-exceptions on Linux
    test_jobs += _generate_jobs(
        languages=["c", "c++"],
        configs=["noexcept"],
        platforms=["linux"],
        labels=["portability", "corelang"],
        extra_args=extra_args,
        inner_jobs=inner_jobs,
        timeout_seconds=_CPP_RUNTESTS_TIMEOUT,
    )

    test_jobs += _generate_jobs(
        languages=["python"],
        configs=["dbg"],
        platforms=["linux"],
        arch="default",
        compiler="python_alpine",
        labels=["portability", "multilang"],
        extra_args=extra_args + ["--report_multi_target"],
        inner_jobs=inner_jobs,
    )

    return test_jobs


def _allowed_labels():
    """Returns a list of existing job labels."""
    all_labels = set()
    for job in _create_test_jobs() + _create_portability_test_jobs():
        for label in job.labels:
            all_labels.add(label)
    return sorted(all_labels)


def _runs_per_test_type(arg_str):
    """Auxiliary function to parse the "runs_per_test" flag."""
    try:
        n = int(arg_str)
        if n <= 0:
            raise ValueError
        return n
    except:
        msg = "'{}' is not a positive integer".format(arg_str)
        raise argparse.ArgumentTypeError(msg)


if __name__ == "__main__":
    argp = argparse.ArgumentParser(
        description="Run a matrix of run_tests.py tests."
    )
    argp.add_argument(
        "-j",
        "--jobs",
        default=multiprocessing.cpu_count() / _DEFAULT_INNER_JOBS,
        type=int,
        help="Number of concurrent run_tests.py instances.",
    )
    argp.add_argument(
        "-f",
        "--filter",
        choices=_allowed_labels(),
        nargs="+",
        default=[],
        help="Filter targets to run by label with AND semantics.",
    )
    argp.add_argument(
        "--exclude",
        choices=_allowed_labels(),
        nargs="+",
        default=[],
        help="Exclude targets with any of given labels.",
    )
    argp.add_argument(
        "--build_only",
        default=False,
        action="store_const",
        const=True,
        help="Pass --build_only flag to run_tests.py instances.",
    )
    argp.add_argument(
        "--force_default_poller",
        default=False,
        action="store_const",
        const=True,
        help="Pass --force_default_poller to run_tests.py instances.",
    )
    argp.add_argument(
        "--dry_run",
        default=False,
        action="store_const",
        const=True,
        help="Only print what would be run.",
    )
    argp.add_argument(
        "--filter_pr_tests",
        default=False,
        action="store_const",
        const=True,
        help="Filters out tests irrelevant to pull request changes.",
    )
    argp.add_argument(
        "--base_branch",
        default="origin/master",
        type=str,
        help="Branch that pull request is requesting to merge into",
    )
    argp.add_argument(
        "--inner_jobs",
        default=_DEFAULT_INNER_JOBS,
        type=int,
        help="Number of jobs in each run_tests.py instance",
    )
    argp.add_argument(
        "-n",
        "--runs_per_test",
        default=1,
        type=_runs_per_test_type,
        help="How many times to run each tests. >1 runs implies "
        + "omitting passing test from the output & reports.",
    )
    argp.add_argument(
        "--max_time",
        default=-1,
        type=int,
        help="Maximum amount of time to run tests for"
        + "(other tests will be skipped)",
    )
    argp.add_argument(
        "--internal_ci",
        default=False,
        action="store_const",
        const=True,
        help=(
            "(Deprecated, has no effect) Put reports into subdirectories to"
            " improve presentation of results by Kokoro."
        ),
    )
    argp.add_argument(
        "--bq_result_table",
        default="",
        type=str,
        nargs="?",
        help="Upload test results to a specified BQ table.",
    )
    argp.add_argument(
        "--extra_args",
        default="",
        type=str,
        nargs=argparse.REMAINDER,
        help="Extra test args passed to each sub-script.",
    )
    args = argp.parse_args()

    extra_args = []
    if args.build_only:
        extra_args.append("--build_only")
    if args.force_default_poller:
        extra_args.append("--force_default_poller")
    if args.runs_per_test > 1:
        extra_args.append("-n")
        extra_args.append("%s" % args.runs_per_test)
        extra_args.append("--quiet_success")
    if args.max_time > 0:
        extra_args.extend(("--max_time", "%d" % args.max_time))
    if args.bq_result_table:
        extra_args.append("--bq_result_table")
        extra_args.append("%s" % args.bq_result_table)
        extra_args.append("--measure_cpu_costs")
    if args.extra_args:
        extra_args.extend(args.extra_args)

    all_jobs = _create_test_jobs(
        extra_args=extra_args, inner_jobs=args.inner_jobs
    ) + _create_portability_test_jobs(
        extra_args=extra_args, inner_jobs=args.inner_jobs
    )

    jobs = []
    for job in all_jobs:
        if not args.filter or all(
            filter in job.labels for filter in args.filter
        ):
            if not any(
                exclude_label in job.labels for exclude_label in args.exclude
            ):
                jobs.append(job)

    if not jobs:
        jobset.message(
            "FAILED", "No test suites match given criteria.", do_newline=True
        )
        sys.exit(1)

    print("IMPORTANT: The changes you are testing need to be locally committed")
    print("because only the committed changes in the current branch will be")
    print("copied to the docker environment or into subworkspaces.")

    skipped_jobs = []

    if args.filter_pr_tests:
        print("Looking for irrelevant tests to skip...")
        relevant_jobs = filter_tests(jobs, args.base_branch)
        if len(relevant_jobs) == len(jobs):
            print("No tests will be skipped.")
        else:
            print("These tests will be skipped:")
            skipped_jobs = list(set(jobs) - set(relevant_jobs))
            # Sort by shortnames to make printing of skipped tests consistent
            skipped_jobs.sort(key=lambda job: job.shortname)
            for job in list(skipped_jobs):
                print("  %s" % job.shortname)
        jobs = relevant_jobs

    print("Will run these tests:")
    for job in jobs:
        print('  %s: "%s"' % (job.shortname, " ".join(job.cmdline)))
    print("")

    if args.dry_run:
        print("--dry_run was used, exiting")
        sys.exit(1)

    jobset.message("START", "Running test matrix.", do_newline=True)
    num_failures, resultset = jobset.run(
        jobs, newline_on_success=True, travis=True, maxjobs=args.jobs
    )
    # Merge skipped tests into results to show skipped tests on report.xml
    if skipped_jobs:
        ignored_num_skipped_failures, skipped_results = jobset.run(
            skipped_jobs, skip_jobs=True
        )
        resultset.update(skipped_results)
    report_utils.render_junit_xml_report(
        resultset,
        _report_filename(_MATRIX_REPORT_NAME),
        suite_name=_MATRIX_REPORT_NAME,
        multi_target=True,
    )

    if num_failures == 0:
        jobset.message(
            "SUCCESS",
            "All run_tests.py instances finished successfully.",
            do_newline=True,
        )
    else:
        jobset.message(
            "FAILED",
            "Some run_tests.py instances have failed.",
            do_newline=True,
        )
        sys.exit(1)



#!/usr/bin/env python2.7
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
import sys
import yaml

run_dir = os.path.dirname(sys.argv[0])
sources_path = os.path.abspath(
    os.path.join(run_dir, "../../third_party/boringssl-with-bazel/sources.json")
)
try:
    with open(sources_path, "r") as s:
        sources = json.load(s)
except IOError:
    sources_path = os.path.abspath(
        os.path.join(
            run_dir, "../../../../third_party/openssl/boringssl/sources.json"
        )
    )
    with open(sources_path, "r") as s:
        sources = json.load(s)


def map_dir(filename):
    return "third_party/boringssl-with-bazel/" + filename


class Grpc(object):
    """Adapter for boring-SSL json sources files."""

    def __init__(self, sources):
        self.yaml = None
        self.WriteFiles(sources)

    def WriteFiles(self, files):
        test_binaries = ["ssl_test", "crypto_test"]
        asm_outputs = {
            key: value
            for key, value in files.items()
            if any(f.endswith(".S") or f.endswith(".asm") for f in value)
        }
        self.yaml = {
            "#": "generated with src/boringssl/gen_build_yaml.py",
            "raw_boringssl_build_output_for_debugging": {
                "files": files,
            },
            "libs": [
                {
                    "name": "boringssl",
                    "build": "private",
                    "language": "c",
                    "secure": False,
                    "src": sorted(
                        map_dir(f) for f in files["ssl"] + files["crypto"]
                    ),
                    "asm_src": {
                        k: [map_dir(f) for f in value]
                        for k, value in asm_outputs.items()
                    },
                    "headers": sorted(
                        map_dir(f)
                        # We want to include files['fips_fragments'], but not build them as objects.
                        # See https://boringssl-review.googlesource.com/c/boringssl/+/16946
                        for f in files["ssl_headers"]
                        + files["ssl_internal_headers"]
                        + files["crypto_headers"]
                        + files["crypto_internal_headers"]
                        + files["fips_fragments"]
                    ),
                    "boringssl": True,
                    "defaults": "boringssl",
                },
                {
                    "name": "boringssl_test_util",
                    "build": "private",
                    "language": "c++",
                    "secure": False,
                    "boringssl": True,
                    "defaults": "boringssl",
                    "src": [map_dir(f) for f in sorted(files["test_support"])],
                },
            ],
            "targets": [
                {
                    "name": "boringssl_%s" % test,
                    "build": "test",
                    "run": False,
                    "secure": False,
                    "language": "c++",
                    "src": sorted(map_dir(f) for f in files[test]),
                    "vs_proj_dir": "test/boringssl",
                    "boringssl": True,
                    "defaults": "boringssl",
                    "deps": [
                        "boringssl_test_util",
                        "boringssl",
                    ],
                }
                for test in test_binaries
            ],
            "tests": [
                {
                    "name": "boringssl_%s" % test,
                    "args": [],
                    "exclude_configs": ["asan", "ubsan"],
                    "ci_platforms": ["linux", "mac", "posix", "windows"],
                    "platforms": ["linux", "mac", "posix", "windows"],
                    "flaky": False,
                    "gtest": True,
                    "language": "c++",
                    "boringssl": True,
                    "defaults": "boringssl",
                    "cpu_cost": 1.0,
                }
                for test in test_binaries
            ],
        }


grpc_platform = Grpc(sources)
print(yaml.dump(grpc_platform.yaml))
